@@ -12,8 +12,7 @@ A basic wrapper for the Steam API and its Community Managers.
 __title__ = "steam"
 __author__ = "Gobot1234"
 __license__ = "MIT"
-__version__ = "0.3.2"
-
+__version__ = "0.4.1"
 
 import logging
 from typing import NamedTuple
@@ -33,6 +32,7 @@ from .image import *
 from .invite import *
 from .message import *
 from .models import *
+from .role import *
 from .trade import *
 from .user import *
 
@@ -40,10 +40,13 @@ from .user import *
 class VersionInfo(NamedTuple):
     major: int
     minor: int
-    macro: int
+    micro: int
     releaselevel: str
 
 
-version_info = VersionInfo(major=0, minor=3, macro=2, releaselevel="full")
+version_info = VersionInfo(major=0, minor=4, micro=1, releaselevel="full")
 
 logging.getLogger(__name__).addHandler(logging.NullHandler())
+
+# not for export
+del logging, NamedTuple, VersionInfo
