@@ -28,28 +28,51 @@ This contains a copy of
 https://github.com/ValvePython/steam/blob/master/steam/steamid.py
 """
 
+from __future__ import annotations
+
 import abc
 import asyncio
 import re
 from datetime import datetime
-from typing import TYPE_CHECKING, Awaitable, Callable, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Callable, Coroutine, NoReturn, Optional, Tuple, Union, overload
+
+from typing_extensions import Final, TypedDict
 
 from .badge import UserBadges
 from .comment import Comment
-from .enums import EInstanceFlag, EPersonaState, EPersonaStateFlag, EResult, EType, ETypeChar, EUniverse
+from .enums import (
+    ECommunityVisibilityState,
+    EInstanceFlag,
+    EPersonaState,
+    EPersonaStateFlag,
+    EResult,
+    EType,
+    ETypeChar,
+    EUniverse,
+)
 from .errors import WSException
 from .game import Game
 from .iterators import CommentsIterator
-from .models import URL, Ban
+from .models import Ban, community_route
 from .trade import Inventory
-from .utils import _INVITE_HEX, _INVITE_MAPPING, make_steam64, steam64_from_url
+from .utils import (
+    _INVITE_HEX,
+    _INVITE_MAPPING,
+    ETypeType,
+    EUniverseType,
+    InstanceType,
+    IntOrStr,
+    id64_from_url,
+    make_id64,
+)
 
 if TYPE_CHECKING:
+    import betterproto
     from aiohttp import ClientSession
 
     from .clan import Clan
-    from .client import EventType
     from .group import Group
+    from .http import StrOrURL
     from .image import Image
     from .state import ConnectionState
     from .user import User
@@ -61,121 +84,138 @@ __all__ = (
 )
 
 
+class UserDict(TypedDict):
+    steamid: str
+    personaname: str
+    steamid: str
+    primaryclanid: str
+    profileurl: str
+    realname: str
+    # enums
+    communityvisibilitystate: int
+    profilestate: int
+    commentpermission: int
+    personastate: int
+    personastateflags: int
+    # avatar
+    avatar: str
+    avatarmedium: str
+    avatarfull: str
+    avatarhash: str
+    # country stuff
+    loccountrycode: str
+    locstatecode: int
+    loccityid: int
+    # game stuff
+    gameid: str  # game app id
+    gameextrainfo: str  # game name
+    # unix timestamps
+    timecreated: int
+    lastlogoff: int
+    # we pass these ourselves
+    last_logon: int
+    last_seen_online: int
+
+
 class SteamID(metaclass=abc.ABCMeta):
     """Convert a Steam ID between its various representations."""
 
-    __slots__ = ("_BASE", "__weakref__")
+    __slots__ = (
+        "__BASE",
+        "__weakref__",
+    )
 
-    def __init__(self, *args, **kwargs):
-        self._BASE = make_steam64(*args, **kwargs)
+    @overload
+    def __init__(self):
+        ...
 
-    def __repr__(self):
-        attrs = ("id", "type", "universe", "instance")
-        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
-        return f"<SteamID {' '.join(resolved)}>"
+    @overload
+    def __init__(
+        self,
+        id: IntOrStr = 0,
+        type: Optional[ETypeType] = None,
+        universe: Optional[EUniverseType] = None,
+        instance: Optional[InstanceType] = None,
+    ):
+        ...
+
+    def __init__(
+        self,
+        id: IntOrStr = 0,
+        type: Optional[ETypeType] = None,
+        universe: Optional[EUniverseType] = None,
+        instance: Optional[InstanceType] = None,
+    ):
+        self.__BASE: Final[int] = make_id64(id, type, universe, instance)
 
     def __int__(self):
-        return self._BASE
+        return self.__BASE
 
-    def __bool__(self):
-        return bool(self._BASE)
+    def __eq__(self, other: Any) -> bool:
+        try:
+            return int(self) == int(other)
+        except (TypeError, ValueError):
+            return NotImplemented
 
-    def __str__(self):
-        return str(self._BASE)
+    def __str__(self) -> str:
+        return str(self.__BASE)
 
-    def __hash__(self):
-        return hash(self._BASE)
+    def __hash__(self) -> int:
+        return hash(self.__BASE)
 
-    def __eq__(self, other):
-        return self._BASE == other
-
-    @property
-    def id(self) -> int:
-        """:class:`int`: Represents the account id.
-        This is also known as the 32 bit id.
-        """
-        return int(self) & 0xFFFFFFFF
+    def __repr__(self) -> str:
+        return f"SteamID(id={self.id}, type={self.type}, universe={self.universe}, instance={self.instance})"
 
     @property
     def instance(self) -> int:
-        """:class:`int`: Returns the instance of the account."""
+        """:class:`int`: The instance of the SteamID."""
         return (int(self) >> 32) & 0xFFFFF
 
     @property
     def type(self) -> EType:
-        """:class:`~steam.EType`: Represents the Steam type of the account."""
+        """:class:`~steam.EType`: The Steam type of the SteamID."""
         return EType((int(self) >> 52) & 0xF)
 
     @property
     def universe(self) -> EUniverse:
-        """:class:`~steam.EUniverse`: Represents the Steam universe of the account."""
+        """:class:`~steam.EUniverse`: The Steam universe of the SteamID."""
         return EUniverse((int(self) >> 56) & 0xFF)
 
-    @property
-    def as_32(self) -> int:
-        """:class:`int`: The account's id.
-        An alias to :attr:`SteamID.id`.
-        """
-        return self.id
-
     @property
     def id64(self) -> int:
-        """:class:`int`: The steam 64 bit id of the account.
-        Used for community profiles along with other useful things.
-        """
+        """:class:`int`: The SteamID's 64 bit ID."""
         return int(self)
 
     @property
-    def as_64(self) -> int:
-        """:class:`int`: The steam 64 bit id of the account.
-        An alias to :attr:`id64`.
-        """
-        return self.id64
+    def id(self) -> int:
+        """:class:`int`: The SteamID's 32 bit ID."""
+        return int(self) & 0xFFFFFFFF
 
     @property
     def id2(self) -> str:
-        """:class:`str`: The Steam2 id of the account.
-            e.g ``STEAM_1:0:1234``.
+        """:class:`str`: The SteamID's ID 2.
 
-        .. note::
-            ``STEAM_X:Y:Z``. The value of ``X`` should represent the universe, or ``1``
-            for ``Public``. However, there was a bug in GoldSrc and Orange Box games
-            and ``X`` was ``0``. If you need that format use :attr:`as_steam2_zero`.
+        e.g ``STEAM_1:0:1234``.
         """
         return f"STEAM_{int(self.universe)}:{self.id % 2}:{self.id >> 1}"
 
     @property
-    def as_steam2(self) -> str:
-        """:class:`str`: The Steam2 id of the account.
-            e.g ``STEAM_1:0:1234``.
+    def id2_zero(self) -> str:
+        """:class:`str`: The SteamID's ID 2 accounted for bugged GoldSrc and Orange Box games. In these games the
+        accounts :attr:`universe`, ``1`` for :class:`.EType.Public`, should be the ``X`` component of ``STEAM_X:0:1234``
+        however, this was bugged and the value of ``X`` was ``0``.
 
-        .. note::
-            ``STEAM_X:Y:Z``. The value of ``X`` should represent the universe, or ``1``
-            for ``Public``. However, there was a bug in GoldSrc and Orange Box games
-            and ``X`` was ``0``. If you need that format use :attr:`as_steam2_zero`.
-
-        An alias to :attr:`id2`.
+        e.g ``STEAM_0:0:1234``.
         """
-        return self.id2
-
-    @property
-    def as_steam2_zero(self) -> str:
-        """:class:`str`: The Steam2 id of the account.
-            e.g ``STEAM_0:0:1234``.
-
-        For GoldSrc and Orange Box games.
-        See :attr:`id2`.
-        """
-        return self.as_steam2.replace("_1", "_0")
+        return self.id2.replace("_1", "_0")
 
     @property
     def id3(self) -> str:
-        """:class:`str`: The Steam3 id of the account.
-            e.g ``[U:1:1234]``.
+        """:class:`str`: The SteamID's ID 3.
 
-        This is used for more recent games.
+        e.g ``[U:1:1234]``.
         """
-        typechar = ETypeChar(self.type).name
+        type_char = ETypeChar(self.type).name
         instance = None
 
         if self.type in (EType.AnonGameServer, EType.Multiseat):
@@ -185,69 +225,56 @@ class SteamID(metaclass=abc.ABCMeta):
                 instance = self.instance
         elif self.type == EType.Chat:
             if self.instance & EInstanceFlag.Clan:
-                typechar = "c"
+                type_char = "c"
             elif self.instance & EInstanceFlag.Lobby:
-                typechar = "L"
+                type_char = "L"
             else:
-                typechar = "T"
+                type_char = "T"
 
-        parts = [typechar, int(self.universe), self.id]
+        parts = [type_char, int(self.universe), self.id]
 
         if instance is not None:
             parts.append(instance)
 
         return f'[{":".join(map(str, parts))}]'
 
-    @property
-    def as_steam3(self) -> str:
-        """:class:`str`: The Steam3 id of the account.
-        An alias to :attr:`id3`.
-        """
-        return self.id3
-
     @property
     def invite_code(self) -> Optional[str]:
-        """Optional[:class:`str`]: s.team invite code format.
-        e.g. ``cv-dgb``
+        """Optional[:class:`str`]: The SteamID's invite code in the s.team invite code format.
+
+        e.g. ``cv-dgb``.
         """
         if self.type == EType.Individual and self.is_valid():
-
-            def repl_mapper(x):
-                return _INVITE_MAPPING[x.group()]
-
-            invite_code = re.sub(f"[{_INVITE_HEX}]", repl_mapper, f"{self.id:x}")
+            invite_code = re.sub(f"[{_INVITE_HEX}]", lambda x: _INVITE_MAPPING[x.group()], f"{self.id:x}")
             split_idx = len(invite_code) // 2
-
-            if split_idx:
-                invite_code = f"{invite_code[:split_idx]}-{invite_code[split_idx:]}"
-
-            return invite_code
+            return invite_code if split_idx == 0 else f"{invite_code[:split_idx]}-{invite_code[split_idx:]}"
 
     @property
     def invite_url(self) -> Optional[str]:
-        """Optional[:class:`str`]: The user's full invite code URL.
-        e.g ``https://s.team/p/cv-dgb``
+        """Optional[:class:`str`]: The SteamID's full invite code URL.
+
+        e.g ``https://s.team/p/cv-dgb``.
         """
         code = self.invite_code
-        if code:
-            return f"https://s.team/p/{code}"
+        return f"https://s.team/p/{code}" if code else None
 
     @property
     def community_url(self) -> Optional[str]:
-        """Optional[:class:`str`]: The community url of the account
+        """Optional[:class:`str`]: The SteamID's community url.
+
         e.g https://steamcommunity.com/profiles/123456789.
         """
         suffix = {
             EType.Individual: "profiles",
             EType.Clan: "gid",
         }
-        if self.type in suffix:
+        try:
             return f"https://steamcommunity.com/{suffix[self.type]}/{self.id64}"
+        except KeyError:
+            return None
 
     def is_valid(self) -> bool:
-        """:class:`bool`: Check whether this SteamID is valid.
-        This doesn't however mean that a matching profile can be found.
-        """
+        """:class:`bool`: Whether or not the SteamID would be valid."""
         if self.type == EType.Invalid or self.type >= EType.Max:
             return False
 
@@ -274,31 +301,102 @@ class SteamID(metaclass=abc.ABCMeta):
 
     @classmethod
     async def from_url(
-        cls, url: str, session: Optional["ClientSession"] = None, timeout: float = 30
-    ) -> Optional["SteamID"]:
-        """Takes Steam community url and returns a SteamID instance or ``None``.
-        See :func:`steam64_from_url` for details.
+        cls, url: StrOrURL, session: Optional[ClientSession] = None, timeout: float = 30
+    ) -> Optional[SteamID]:
+        """|coro|
+        A helper function creates a SteamID instance from a Steam community url. See :func:`id64_from_url` for details.
 
         Parameters
         ----------
-        url: :class:`str`
-            The Steam community url.
+        url: Union[:class:`str`, :class:`yarl.URL`]
+            The Steam community url to fetch.
         session: Optional[:class:`aiohttp.ClientSession`]
-            The session to make the request with. If
-            ``None`` is passed a new one is generated.
-        timeout: Optional[:class:`float`]
-            How long to wait on http request before turning ``None``.
+            The session to make the request with. If ``None`` is passed a new one is generated.
+        timeout: :class:`float`
+            How long to wait for a response before returning ``None``.
 
         Returns
         -------
         Optional[:class:`SteamID`]
-            `SteamID` instance or ``None``.
+            :class:`SteamID` instance or ``None``.
         """
-        id64 = await steam64_from_url(url, session, timeout)
+        id64 = await id64_from_url(url, session, timeout)
         return cls(id64) if id64 else None
 
 
-class BaseUser(SteamID):
+class Commentable(SteamID):
+    """A mixin that implements commenting functionality"""
+
+    __slots__ = ("comment_path",)
+
+    _state: ConnectionState
+    comment_path: str
+
+    def __init_subclass__(cls, **kwargs: Any):
+        cls.comment_path: Final[str] = kwargs.get("comment_path", "Profile")
+
+    async def comment(self, content: str) -> Comment:
+        """|coro|
+        Post a comment to a profile.
+
+        Parameters
+        -----------
+        content: :class:`str`
+            The message to add to the profile.
+
+        Returns
+        -------
+        :class:`~steam.Comment`
+            The created comment.
+        """
+        resp = await self._state.http.post_comment(self.id64, self.comment_path, content)
+        id = int(re.findall(r'id="comment_(\d+)"', resp["comments_html"])[0])
+        timestamp = datetime.utcfromtimestamp(resp["timelastpost"])
+        comment = Comment(
+            state=self._state, id=id, owner=self, timestamp=timestamp, content=content, author=self._state.client.user
+        )
+        self._state.dispatch("comment", comment)
+        return comment
+
+    def comments(
+        self, limit: Optional[int] = None, before: Optional[datetime] = None, after: Optional[datetime] = None
+    ) -> CommentsIterator:
+        """An :class:`~steam.iterators.AsyncIterator` for accessing a profiles's :class:`~steam.Comment` objects.
+
+        Examples
+        ---------
+
+        Usage: ::
+
+            async for comment in commentable.comments(limit=10):
+                print('Author:', comment.author, 'Said:', comment.content)
+
+        Flattening into a list: ::
+
+            comments = await commentable.comments(limit=50).flatten()
+            # comments is now a list of Comment
+
+        All parameters are optional.
+
+        Parameters
+        ----------
+        limit: Optional[:class:`int`]
+            The maximum number of comments to search through.
+            Default is ``None`` which will fetch the clan's entire comments section.
+        before: Optional[:class:`datetime.datetime`]
+            A time to search for comments before.
+        after: Optional[:class:`datetime.datetime`]
+            A time to search for comments after.
+
+        Yields
+        ---------
+        :class:`~steam.Comment`
+            The comment with the comment information parsed.
+        """
+        return CommentsIterator(state=self._state, owner=self, limit=limit, before=before, after=after)
+
+
+class BaseUser(Commentable):
     """An ABC that details the common operations on a Steam user.
     The following classes implement this ABC:
 
@@ -326,8 +424,8 @@ class BaseUser(SteamID):
     state: :class:`~steam.EPersonaState`
         The current persona state of the account (e.g. LookingToTrade).
     game: Optional[:class:`~steam.Game`]
-        The Game instance attached to the user. Is None if the user
-        isn't in a game or one that is recognised by the api.
+        The Game instance attached to the user. Is ``None`` if the user isn't in a game or one that is recognised by the
+        api.
     primary_clan: Optional[:class:`SteamID`]
         The primary clan the User displays on their profile.
     avatar_url: :class:`str`
@@ -340,7 +438,7 @@ class BaseUser(SteamID):
         The last time the user logged into steam. Could be None (e.g. if they are currently online).
     country: Optional[:class:`str`]
         The country code of the account. Could be ``None``.
-    flags: Union[:class:`~steam.EPersonaStateFlag`, :class:`int`]
+    flags: list[:class:`~steam.EPersonaStateFlag`]
         The persona state flags of the account.
     """
 
@@ -358,11 +456,13 @@ class BaseUser(SteamID):
         "created_at",
         "last_logoff",
         "last_logon",
+        "privacy_state",
         "_state",
-        "_data",
+        "_is_commentable",
+        "_setup_profile",
     )
 
-    def __init__(self, state: "ConnectionState", data: dict):
+    def __init__(self, state: ConnectionState, data: UserDict):
         super().__init__(data["steamid"])
         self._state = state
         self.name: Optional[str] = None
@@ -374,70 +474,79 @@ class BaseUser(SteamID):
         self.last_logoff: Optional[datetime] = None
         self.last_logon: Optional[datetime] = None
         self.last_seen_online: Optional[datetime] = None
-        self.state: Optional[EPersonaState] = None
-        self.flags: Optional[EPersonaStateFlag] = None
         self.game: Optional[Game] = None
+        self.state: Optional[EPersonaState] = None
+        self.flags: list[EPersonaStateFlag] = []
+        self.privacy_state: Optional[ECommunityVisibilityState] = None
         self._update(data)
 
-    def __repr__(self):
+    def _update(self, data: UserDict) -> None:
+        self.name = data["personaname"]
+        self.real_name = data.get("realname") or self.real_name
+        self.avatar_url = data.get("avatarfull") or self.avatar_url
+        self.trade_url = community_route(f"tradeoffer/new/?partner={self.id}")
+
+        self.primary_clan = SteamID(data["primaryclanid"]) if "primaryclanid" in data else self.primary_clan
+        self.country = data.get("loccountrycode") or self.country
+        self.created_at = datetime.utcfromtimestamp(data["timecreated"]) if "timecreated" in data else self.created_at
+        self.last_logoff = datetime.utcfromtimestamp(data["lastlogoff"]) if "lastlogoff" in data else self.last_logoff
+        self.last_logon = datetime.utcfromtimestamp(data["last_logon"]) if "last_logon" in data else self.last_logon
+        self.last_seen_online = (
+            datetime.utcfromtimestamp(data["last_seen_online"]) if "last_seen_online" in data else self.last_seen_online
+        )
+        self.game = Game(title=data.get("gameextrainfo"), id=data["gameid"]) if "gameid" in data else self.game
+        self.state = EPersonaState(data.get("personastate", 0)) or self.state
+        self.flags = EPersonaStateFlag.components(data.get("personastateflags", 0)) or self.flags
+        self.privacy_state = ECommunityVisibilityState(data.get("communityvisibilitystate", 0))
+        self._is_commentable = bool(data.get("commentpermission"))
+        self._setup_profile = bool(data.get("profilestate"))
+
+    def __repr__(self) -> str:
         attrs = ("name", "state", "id", "type", "universe", "instance")
         resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
         return f"<User {' '.join(resolved)}>"
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
-    def _update(self, data) -> None:
-        self._data = data
-        self.name = data["personaname"]
-        self.real_name = data.get("realname") or self.real_name
-        self.avatar_url = data.get("avatarfull") or self.avatar_url
-        self.trade_url = f"{URL.COMMUNITY}/tradeoffer/new/?partner={self.id}"
+    def __copy__(self) -> BaseUser:
+        # can't use default implementation due to comment_path being read only
+        flag_value = 0
+        for flag in self.flags:
+            flag_value |= flag
+        data = {
+            "steamid": self.id64,
+            "personaname": self.name,
+            "realname": self.real_name,
+            "avatarfull": self.avatar_url,
+            "primaryclanid": self.primary_clan,
+            "loccountrycode": self.country,
+            "gameextrainfo": self.game.title if self.game else None,
+            "personastate": self.state,
+            "personastateflags": flag_value,
+            "communityvisibilitystate": self.privacy_state,
+            "commentpermission": self._is_commentable,
+            "profilestate": self._setup_profile,
+        }
+        if self.game is not None:
+            data.update({"gameid": self.game.id})
+        if self.created_at is not None:
+            data.update({"timecreated": self.created_at.timestamp()})
+        if self.last_logoff is not None:
+            data.update({"lastlogoff": self.last_logoff.timestamp()})
+        if self.last_logon is not None:
+            data.update({"last_logon": self.last_logon.timestamp()})
+        if self.last_seen_online is not None:
+            data.update({"last_seen_online": self.last_seen_online.timestamp()})
 
-        self.primary_clan = SteamID(data["primaryclanid"]) if "primaryclanid" in data else None or self.primary_clan
-        self.country = data.get("loccountrycode") or self.country
-        self.created_at = (
-            datetime.utcfromtimestamp(data["timecreated"]) if "timecreated" in data else None or self.created_at
-        )
-        self.last_logoff = (
-            datetime.utcfromtimestamp(data["lastlogoff"]) if "lastlogoff" in data else None or self.last_logoff
-        )
-        self.last_logon = (
-            datetime.utcfromtimestamp(data["last_logon"]) if "last_logon" in data else None or self.last_logon
-        )
-        self.last_seen_online = (
-            datetime.utcfromtimestamp(data["last_seen_online"])
-            if "last_seen_online" in data
-            else None or self.last_seen_online
-        )
-        self.game = (
-            Game(title=data.get("gameextrainfo"), app_id=data["gameid"]) if "gameid" in data else None or self.game
-        )
-        self.state = EPersonaState(data.get("personastate", 0)) or self.state
-        self.flags = EPersonaStateFlag.try_value(data.get("personastateflags", 0)) or self.flags
+        return self.__class__(state=self._state, data=data)
 
-    async def comment(self, content: str) -> Comment:
-        """|coro|
-        Post a comment to an :class:`User`'s profile.
+    copy = __copy__
 
-        Parameters
-        -----------
-        content: :class:`str`
-            The message to add to the user's profile.
-
-        Returns
-        -------
-        :class:`~steam.Comment`
-            The created comment.
-        """
-        resp = await self._state.http.post_comment(self.id64, "Profile", content)
-        id = int(re.findall(r'id="comment_(\d+)"', resp["comments_html"])[0])
-        timestamp = datetime.utcfromtimestamp(resp["timelastpost"])
-        comment = Comment(
-            state=self._state, id=id, owner=self, timestamp=timestamp, content=content, author=self._state.client.user,
-        )
-        self._state.dispatch("comment", comment)
-        return comment
+    @property
+    def mention(self) -> str:
+        """:class:`str`: The string used to mention the user."""
+        return f"[mention={self.id}]@{self.name}[/mention]"
 
     async def inventory(self, game: Game) -> Inventory:
         """|coro|
@@ -458,52 +567,51 @@ class BaseUser(SteamID):
         :class:`Inventory`
             The user's inventory.
         """
-        resp = await self._state.http.get_user_inventory(self.id64, game.app_id, game.context_id)
+        resp = await self._state.http.get_user_inventory(self.id64, game.id, game.context_id)
         return Inventory(state=self._state, data=resp, owner=self)
 
-    async def friends(self) -> List["User"]:
+    async def friends(self) -> list[User]:
         """|coro|
         Fetch the list of :class:`~steam.User`'s friends from the API.
 
         Returns
         -------
-        List[:class:`~steam.User`]
+        list[:class:`~steam.User`]
             The list of user's friends from the API.
         """
         friends = await self._state.http.get_friends(self.id64)
         return [self._state._store_user(friend) for friend in friends]
 
-    async def games(self) -> List[Game]:
+    async def games(self) -> list[Game]:
         """|coro|
-        Fetches the list of :class:`~steam.Game`
-        objects the :class:`User` owns from the API.
+        Fetches the :class:`~steam.Game` objects the :class:`User` owns from the API.
 
         Returns
         -------
-        List[:class:`~steam.Game`]
+        list[:class:`~steam.Game`]
             The list of game objects from the API.
         """
         data = await self._state.http.get_user_games(self.id64)
         games = data["response"].get("games", [])
         return [Game._from_api(game) for game in games]
 
-    async def clans(self) -> List["Clan"]:
+    async def clans(self) -> list[Clan]:
         """|coro|
         Fetches a list of the :class:`User`'s :class:`~steam.Clan`
         objects the :class:`User` is in from the API.
 
         Returns
         -------
-        List[:class:`~steam.Clan`]
+        list[:class:`~steam.Clan`]
             The user's clans.
         """
         clans = []
 
-        async def getter(gid: int):
+        async def getter(gid: int) -> None:
             try:
                 clan = await self._state.client.fetch_clan(gid)
             except WSException as exc:
-                if exc.code == (EResult.LimitExceeded, EResult.RateLimitExceeded):
+                if exc.code == EResult.RateLimitExceeded:
                     await asyncio.sleep(20)
                     await getter(gid)
             else:
@@ -525,7 +633,7 @@ class BaseUser(SteamID):
         """
         resp = await self._state.http.get_user_bans(self.id64)
         resp = resp["players"][0]
-        resp["EconomyBan"] = False if resp["EconomyBan"] == "none" else True
+        resp["EconomyBan"] = resp["EconomyBan"] != "none"
         return Ban(data=resp)
 
     async def badges(self) -> UserBadges:
@@ -554,16 +662,15 @@ class BaseUser(SteamID):
 
     def is_commentable(self) -> bool:
         """:class:`bool`: Specifies if the user's account is able to be commented on."""
-        return bool(self._data.get("commentpermission"))
+        return self._is_commentable
 
     def is_private(self) -> bool:
         """:class:`bool`: Specifies if the user has a public profile."""
-        state = self._data.get("communityvisibilitystate", 0)
-        return state in (0, 1, 2)
+        return self.privacy_state == ECommunityVisibilityState.Private
 
     def has_setup_profile(self) -> bool:
         """:class:`bool`: Specifies if the user has a setup their profile."""
-        return bool(self._data.get("profilestate"))
+        return self._setup_profile
 
     async def is_banned(self) -> bool:
         """|coro|
@@ -582,46 +689,8 @@ class BaseUser(SteamID):
         bans = await self.bans()
         return bans.is_banned()
 
-    def comments(
-        self, limit: Optional[int] = None, before: Optional[datetime] = None, after: Optional[datetime] = None,
-    ) -> CommentsIterator:
-        """An :class:`~steam.iterators.AsyncIterator` for accessing a
-        :class:`~steam.User`'s :class:`~steam.Comment` objects.
 
-        Examples
-        -----------
-
-        Usage: ::
-
-            async for comment in user.comments(limit=10):
-                print('Author:', comment.author, 'Said:', comment.content)
-
-        Flattening into a list: ::
-
-            comments = await user.comments(limit=50).flatten()
-            # comments is now a list of Comment
-
-        All parameters are optional.
-
-        Parameters
-        ----------
-        limit: Optional[:class:`int`]
-            The maximum number of comments to search through.
-            Default is ``None`` which will fetch the user's entire comments section.
-        before: Optional[:class:`datetime.datetime`]
-            A time to search for comments before.
-        after: Optional[:class:`datetime.datetime`]
-            A time to search for comments after.
-
-        Yields
-        ---------
-        :class:`~steam.Comment`
-            The comment with the comment information parsed.
-        """
-        return CommentsIterator(state=self._state, owner=self, limit=limit, before=before, after=after)
-
-
-_EndPointReturnType = Tuple[Union[Tuple[int, int], int], "EventType"]
+_EndPointReturnType = Tuple[Union[Tuple[int, int], int], Callable[..., Coroutine[None, None, None]]]
 
 
 class Messageable(metaclass=abc.ABCMeta):
@@ -636,13 +705,15 @@ class Messageable(metaclass=abc.ABCMeta):
 
     __slots__ = ()
 
+    @abc.abstractmethod
     def _get_message_endpoint(self) -> _EndPointReturnType:
-        pass
+        raise NotImplementedError
 
+    @abc.abstractmethod
     def _get_image_endpoint(self) -> _EndPointReturnType:
-        pass
+        raise NotImplementedError
 
-    async def send(self, content: Optional[str] = None, image: Optional["Image"] = None):
+    async def send(self, content: Optional[str] = None, image: Optional[Image] = None) -> None:
         """|coro|
         Send a message to a certain destination.
 
@@ -669,19 +740,25 @@ class Messageable(metaclass=abc.ABCMeta):
 
 
 class BaseChannel(Messageable):
-    __slots__ = ("_state", "participant", "clan", "group")
+    __slots__ = ("clan", "group", "_state")
+
+    _state: "ConnectionState"
 
     def __init__(self):
-        self._state: "ConnectionState"
-        self.participant: Optional["BaseUser"] = None
-        self.clan: Optional["Clan"] = None
-        self.group: Optional["Group"] = None
+        self.clan: Optional[Clan] = None
+        self.group: Optional[Group] = None
 
-    def typing(self):
-        pass
+    @abc.abstractmethod
+    def typing(self) -> NoReturn:
+        raise NotImplementedError
 
-    async def trigger_typing(self):
-        pass
+    @abc.abstractmethod
+    async def trigger_typing(self) -> NoReturn:
+        raise NotImplementedError
+
+
+def _clean_up_content(content: str) -> str:
+    return content.replace(r"\[", "[").replace("\\\\", "\\")
 
 
 class Message:
@@ -696,31 +773,44 @@ class Message:
 
     Attributes
     ----------
-    channel: :class:`steam.abc.BaseChannel`
+    channel: :class:`BaseChannel`
         The channel the message was sent in.
     content: :class:`str`
         The message's content.
-    author: :class:`steam.abc.BaseUser`
+    clean_content: :class:`str`
+        The message's clean content without bbcode.
+    author: :class:`~steam.User`
         The message's author.
     created_at: :class:`datetime.datetime`
         The time the message was sent at.
+    group: Optional[:class:`~steam.Group`]
+        The group the message was sent in. Will be ``None`` if the message wasn't sent in a :class:`~steam.Group`.
+    clan: Optional[:class:`~steam.Clan`]
+        The clan the message was sent in. Will be ``None`` if the message wasn't sent in a :class:`~steam.Clan`.
     """
 
     __slots__ = (
         "author",
         "content",
         "channel",
+        "clean_content",
         "created_at",
         "group",
         "clan",
         "_state",
     )
 
-    def __init__(self, channel: "BaseChannel"):
-        self._state: "ConnectionState" = channel._state
+    def __init__(self, channel: BaseChannel, proto: betterproto.Message):
+        self._state: ConnectionState = channel._state
         self.channel = channel
-        self.content: Optional[str] = None
-        self.author: Optional[BaseUser] = None
-        self.created_at: Optional[datetime] = None
-        self.group = channel.group
-        self.clan = channel.clan
+        self.group: Optional[Group] = channel.group
+        self.clan: Optional[Clan] = channel.clan
+        self.content: str = _clean_up_content(proto.message)
+        self.clean_content: str = proto.message_no_bbcode or self.content
+        self.author: "User"
+        self.created_at: datetime
+
+    def __repr__(self) -> str:
+        attrs = ("author", "channel")
+        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
+        return f"<{self.__class__.__name__} {' '.join(resolved)}>"
