@@ -30,9 +30,10 @@ https://github.com/Rapptz/discord.py/blob/master/discord/gateway.py
 and https://github.com/ValvePython/steam/blob/master/steam/core/cm.py
 """
 
+from __future__ import annotations
+
 import asyncio
 import concurrent.futures
-import functools
 import logging
 import random
 import struct
@@ -42,9 +43,10 @@ import time
 import traceback
 from gzip import GzipFile
 from io import BytesIO
-from typing import TYPE_CHECKING, Callable, Dict, List, NamedTuple, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Callable, NamedTuple, NewType, Optional, Union
 
 import aiohttp
+from typing_extensions import Literal
 
 from . import utils
 from .enums import EPersonaState, EResult
@@ -55,33 +57,50 @@ from .protobufs import EMsg, Msg, MsgProto
 if TYPE_CHECKING:
     from .client import Client
     from .enums import EUIMode
+    from .game import GameDict
     from .protobufs.steammessages_base import CMsgMulti
     from .protobufs.steammessages_clientserver_login import CMsgClientLogonResponse
-    from .state import ConnectionState
+    from .state import ConnectionState, EventParser
 
 
 __all__ = (
     "ConnectionClosed",
     "CMServerList",
     "SteamWebSocket",
+    "return_true",
+    "Msgs",
 )
 
 log = logging.getLogger(__name__)
-EXECUTOR = concurrent.futures.ThreadPoolExecutor()
+Msgs = Union[MsgProto, Msg]
+GoodType = NewType("GoodType", Literal[True])
+BadType = NewType("BadType", Literal[False])
 
 
-def return_true(*_) -> bool:
+def return_true(*_, **__) -> Literal[True]:
     return True
 
 
+def check_job_id(job_id: int) -> Callable[[Msgs], bool]:
+    check = None  # needed to stop pylint thinking this is a NameError
+    exec(f"def check(msg): return msg.header.job_id_target == {job_id}")
+    return check
+
+
 class EventListener(NamedTuple):
     emsg: EMsg
-    predicate: Callable[..., bool]
+    predicate: Callable[[Msgs], bool]
     future: asyncio.Future
 
 
+class CMServer(NamedTuple):
+    url: str
+    state: Union[GoodType, BadType]
+    score: float
+
+
 class ConnectionClosed(Exception):
-    def __init__(self, cm: str, cms: "CMServerList"):
+    def __init__(self, cm: str, cms: CMServerList):
         self.cm = cm
         self.cm_list = cms
         super().__init__(f"Connection to {self.cm}, has closed.")
@@ -92,41 +111,45 @@ class WebSocketClosure(Exception):
 
 
 class CMServerList(AsyncIterator[str]):
-    GOOD = 1
-    BAD = 2
-    __slots__ = ("dict", "cell_id", "best_cms", "_state")
+    GOOD = GoodType(True)
+    BAD = BadType(False)
 
-    def __init__(self, state: "ConnectionState", first_cm_to_try: str):
+    __slots__ = ("cms", "cell_id", "_state")
+
+    def __init__(self, state: ConnectionState, first_cm_to_try: Optional[str] = None):
         super().__init__(state, None, None, None)
-        self.dict: Dict[str, Tuple[int, float]] = dict()
-        self.best_cms: List[Tuple[str, float]] = []
+        self.cms: list[CMServer] = []
         self.cell_id = 0
         if first_cm_to_try is not None:
             self.queue.put_nowait(first_cm_to_try)
 
-    def __len__(self):
-        return len(self.dict)
+    def __len__(self) -> int:
+        return len(self.cms)
+
+    @utils.async_property
+    async def best_cms(self) -> list[CMServer]:
+        good_servers = [cm for cm in self.cms if cm.state]
+
+        if not good_servers:
+            log.debug("No good servers left. Resetting...")
+            self.reset_all()
+            await self.fill()
+            return await self.best_cms
+
+        random.shuffle(good_servers)
+        return await self.ping_cms(good_servers)
 
     async def fill(self) -> None:
         if not await self.fetch_servers_from_api():
             raise NoCMsFound("No Community Managers could be found to connect to")
 
-        good_servers = [k for (k, v) in self.dict.items() if v[0] == self.GOOD]
-
-        if len(good_servers) == 0:
-            log.debug("No good servers left. Resetting...")
-            self.reset_all()
-            return
-
-        random.shuffle(good_servers)
-        await self.ping_cms(good_servers)
-        for server_address, _ in self.best_cms:
-            self.queue.put_nowait(server_address)
+        for cm in await self.best_cms:
+            self.queue.put_nowait(cm.url)
 
     def clear(self) -> None:
-        if self.dict:
+        if self.cms:
             log.debug("List cleared.")
-        self.dict.clear()
+        self.cms = []
 
     async def fetch_servers_from_api(self, cell_id: int = 0) -> bool:
         log.debug("Attempting to fetch servers from the WebAPI")
@@ -134,14 +157,12 @@ class CMServerList(AsyncIterator[str]):
         try:
             resp = await self._state.http.get_cm_list(cell_id)
         except Exception as e:
-            log.error(f"WebAPI fetch request failed with result: {repr(e)}")
+            log.error(f"WebAPI fetch request failed with result: {e!r}")
             return False
 
         resp = resp["response"]
         if resp["result"] != EResult.OK:
-            log.error(
-                f'Fetching the CMList failed with Result: {EResult(resp["result"])} Message: {repr(resp["message"])}'
-            )
+            log.error(f'Fetching the CMList failed with Result: {EResult(resp["result"])} Message: {resp["message"]!r}')
             return False
 
         websockets_list = resp["serverlist_websockets"]
@@ -155,45 +176,44 @@ class CMServerList(AsyncIterator[str]):
 
     def reset_all(self) -> None:
         log.debug("Marking all CM servers as Good.")
-        for server in self.dict:
-            self.mark_good(server)
-        self.best_cms = []
+        for cm in self.cms:
+            self.mark_good(cm.url)
 
-    def mark_good(self, server: str, score: float = 0.0) -> None:
-        self.dict[server] = self.GOOD, score
+    def mark_good(self, url: str, score: float = 0.0) -> None:
+        self.cms.append(CMServer(url, self.GOOD, score))
 
-    def mark_bad(self, server: str) -> None:
-        self.dict[server] = self.BAD, 0.0
+    def mark_bad(self, url: str) -> None:
+        self.cms.append(CMServer(url, self.BAD, 0.0))
 
-    def merge_list(self, hosts: List[str]) -> None:
+    def merge_list(self, hosts: list[str]) -> None:
         total = len(self)
+        urls = [cm.url for cm in self.cms]
         for host in hosts:
-            if host not in self.dict:
+            if host not in urls:
                 self.mark_good(host)
         if len(self) > total:
             log.debug(f"Added {len(self) - total} new CM server addresses.")
 
-    async def ping_cms(self, hosts: Optional[List[str]] = None, to_ping: int = 10) -> None:
-        hosts = list(self.dict.keys()) if hosts is None else hosts
-        for host in hosts[:to_ping]:  # only ping the first 10 cms (by default)
-            # TODO dynamically make sure we get good ones
-            # by checking len and stuff
+    async def ping_cms(self, cms: Optional[list[CMServer]] = None, to_ping: int = 10) -> list[CMServer]:
+        cms = self.cms or cms
+        best_cms = []
+        for cm in cms[:to_ping]:
+            # TODO dynamically make sure we get good ones by checking len and stuff
             start = time.perf_counter()
             try:
-                resp = await self._state.http._session.get(f"https://{host}/cmping/", timeout=5)
+                resp = await self._state.http._session.get(f"https://{cm.url}/cmping/", timeout=5)
                 if resp.status != 200:
-                    self.mark_bad(host)
+                    self.mark_bad(cm.url)
                 load = resp.headers["X-Steam-CMLoad"]
-            except (KeyError, asyncio.TimeoutError):
-                self.mark_bad(host)
+            except (KeyError, asyncio.TimeoutError, aiohttp.ClientError):
+                self.mark_bad(cm.url)
             else:
                 latency = time.perf_counter() - start
                 score = (int(load) * 2) + latency
-                self.best_cms.append((host, score))
-                self.best_cms = sorted(self.best_cms, key=lambda x: x[1])
-                self.mark_good(host, score)
-
+                best_cms.append(cm)
+                self.mark_good(cm.url, score)
         log.debug("Finished pinging CMs")
+        return sorted(best_cms, key=lambda cm: cm.score)
 
 
 class KeepAliveHandler(threading.Thread):  # ping commands are cool
@@ -212,9 +232,9 @@ class KeepAliveHandler(threading.Thread):  # ping commands are cool
         "_main_thread_id",
     )
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs: Any):
         super().__init__()
-        self.ws: "SteamWebSocket" = kwargs.pop("ws")
+        self.ws: SteamWebSocket = kwargs.pop("ws")
         self.interval: int = kwargs.pop("interval")
         self._main_thread_id = self.ws.thread_id
         self.heartbeat = MsgProto(EMsg.ClientHeartBeat)
@@ -242,31 +262,31 @@ class KeepAliveHandler(threading.Thread):  # ping commands are cool
                     return self.stop()
 
             log.debug(self.msg.format(self.heartbeat))
+            if self.ws.loop.is_closed():  # FIXME this is not great
+                return
             coro = self.ws.send_as_proto(self.heartbeat)
             f = asyncio.run_coroutine_threadsafe(coro, loop=self.ws.loop)
-            try:
-                # block until sending is complete
-                total = 0
-                while 1:
+            # block until sending is complete
+            total = 0
+            while 1:
+                try:
+                    f.result(timeout=10)
+                except concurrent.futures.TimeoutError:
+                    total += 10
                     try:
-                        f.result(10)
-                        break
-                    except concurrent.futures.TimeoutError:
-                        total += 10
-                        try:
-                            frame = sys._current_frames()[self._main_thread_id]
-                        except KeyError:
-                            msg = self.block_msg
-                        else:
-                            stack = traceback.format_stack(frame)
-                            msg = f'{self.block_msg}\nLoop thread traceback (most recent call last):\n{"".join(stack)}'
-                        log.warning(msg.format(total))
-
-            except Exception:
-                self.stop()
-            else:
-                self.ack()
-                self._last_send = time.perf_counter()
+                        frame = sys._current_frames()[self._main_thread_id]
+                    except KeyError:
+                        msg = self.block_msg
+                    else:
+                        stack = traceback.format_stack(frame)
+                        msg = f'{self.block_msg}\nLoop thread traceback (most recent call last):\n{"".join(stack)}'
+                    log.warning(msg.format(total))
+                except Exception:
+                    self.stop()
+                else:
+                    self.ack()
+                    self._last_send = time.perf_counter()
+                    break
 
     def stop(self) -> None:
         self._stop_ev.set()
@@ -283,7 +303,6 @@ class SteamWebSocket:
         "socket",
         "loop",
         "cm_list",
-        "cell_id",
         "cm",
         "session_id",
         "thread_id",
@@ -298,22 +317,23 @@ class SteamWebSocket:
         "_keep_alive",
     )
 
-    def __init__(
-        self, socket: aiohttp.ClientWebSocketResponse, *, loop: asyncio.AbstractEventLoop,
-    ):
+    def __init__(self, socket: aiohttp.ClientWebSocketResponse, *, loop: asyncio.AbstractEventLoop):
         self.socket = socket
         self.loop = loop
 
+        # state stuff
         self._connection: Optional["ConnectionState"] = None
         self.cm_list: Optional[CMServerList] = None
+        # the keep alive
         self._keep_alive: Optional[KeepAliveHandler] = None
-        self._dispatch = lambda *args: None
+        # an empty dispatcher to prevent crashes
+        self._dispatch = lambda *args, **kwargs: None
         self.cm: Optional[str] = None
-        self.cell_id = 0
         self.thread_id = threading.get_ident()
 
-        self.listeners: List[EventListener] = []
-        self._parsers: Dict[EMsg, Callable[["ConnectionState", "MsgProto"], None]] = dict()
+        # ws related stuff
+        self.listeners: list[EventListener] = []
+        self._parsers: dict[EMsg, EventParser] = dict()
 
         self.connected = False
         self.session_id = 0
@@ -327,10 +347,10 @@ class SteamWebSocket:
 
     @property
     def latency(self) -> float:
-        """:class:`float`: Measures latency between a HEARTBEAT and a HEARTBEAT_ACK in seconds."""
+        """:class:`float`: Measures latency between a heartbeat send and the heartbeat interval in seconds."""
         return self._keep_alive.latency
 
-    def wait_for(self, emsg: EMsg, predicate: Optional[Callable[..., bool]] = None) -> asyncio.Future:
+    def wait_for(self, emsg: EMsg, predicate: Optional[Callable[[Msgs], bool]] = None) -> asyncio.Future:
         future = self.loop.create_future()
         entry = EventListener(emsg=emsg, predicate=predicate or return_true, future=future)
         self.listeners.append(entry)
@@ -338,13 +358,13 @@ class SteamWebSocket:
 
     @classmethod
     async def from_client(
-        cls, client: "Client", cm: Optional[str] = None, cms: Optional[CMServerList] = None
-    ) -> "SteamWebSocket":
+        cls, client: Client, cm: Optional[str] = None, cms: Optional[CMServerList] = None
+    ) -> SteamWebSocket:
         connection = client._connection
         cm_list = cms or CMServerList(connection, cm)
         async for cm in cm_list:
-            log.info(f"Creating a websocket connection to: {cm}")
-            socket = await client.http.connect_to_cm(cm)
+            log.info(f"Attempting to create a websocket connection to: {cm}")
+            socket: aiohttp.ClientWebSocketResponse = await client.http.connect_to_cm(cm)
             log.debug(f"Connected to {cm}")
             payload = MsgProto(
                 EMsg.ClientLogon,
@@ -372,17 +392,16 @@ class SteamWebSocket:
     async def poll_event(self) -> None:
         try:
             message = await self.socket.receive()
+            if message.type is aiohttp.WSMsgType.BINARY:
+                if message.data:  # it can sometimes be None/empty
+                    return await self.receive(message.data)
             if message.type is aiohttp.WSMsgType.ERROR:
                 log.debug(f"Received {message}")
                 raise message.data
-            if message.type in (aiohttp.WSMsgType.CLOSED, aiohttp.WSMsgType.CLOSE):
+            if message.type in (aiohttp.WSMsgType.CLOSED, aiohttp.WSMsgType.CLOSE, aiohttp.WSMsgType.CLOSING):
                 log.debug(f"Received {message}")
                 raise WebSocketClosure
-
-            message = message.data
-            if not message:  # it can sometimes be None/empty
-                return
-            await self.receive(message)
+            log.debug(f"Dropped unexpected message type: {message}")
         except WebSocketClosure:
             await self.handle_close()
 
@@ -397,27 +416,23 @@ class SteamWebSocket:
             return await self.handlers[emsg](msg)
 
         if not self.connected:
-            return log.debug(f"Dropped unexpected message: {repr(emsg)} {repr(message)}")
+            return log.debug(f"Dropped unexpected message: {emsg} {message!r}")
         try:
-            if utils.is_proto(emsg_value):
-                msg = MsgProto(emsg, message)
-            else:
-                msg = Msg(emsg, message, extended=True)
-            log.debug(f"Socket has received {repr(msg)} from the websocket.")
+            msg = MsgProto(emsg, message) if utils.is_proto(emsg_value) else Msg(emsg, message, extended=True)
+            log.debug(f"Socket has received {msg!r} from the websocket.")
         except Exception as exc:
-            log.critical(f"Failed to deserialize message: {repr(emsg)}, {repr(message)}")
-            if emsg != EMsg.ServiceMethodResponse:  # the repr likely failed so just ignore it
+            try:
+                repr(message)
+            except Exception:
+                log.critical(f"Failed to deserialize message: {emsg!r}, {message!r}")
                 return log.error(exc)
 
-            # log.critical(f"Failed to deserialize message: {repr(emsg)}, {repr(message)}")
-            # log.error(exc)
-
         self._dispatch("socket_receive", msg)
 
         try:
             func = self._parsers[emsg]
         except KeyError:
-            log.debug(f"Ignoring event {repr(msg)}")
+            log.debug(f"Ignoring event {msg!r}")
         else:
             await utils.maybe_coroutine(func, self._connection, msg)
 
@@ -457,27 +472,26 @@ class SteamWebSocket:
         await self.send(bytes(message))
 
     async def close(self, code: int = 4000) -> None:
-        if self._keep_alive is not None:
-            self._keep_alive.stop()
         if self.connected:
             await self.send_as_proto(MsgProto(EMsg.ClientLogOff))
         await self.socket.close(code=code)
 
-    async def handle_close(self):
-        try:
+    async def handle_close(self) -> None:
+        if not self.socket.closed:
             await self.close()
-        except Exception:
-            pass
-        self.cm_list.queue.get_nowait()  # pop the disconnected cm
+            self.cm_list.queue.get_nowait()  # pop the disconnected cm
+        if self._keep_alive is not None:
+            self._keep_alive.stop()
+            self._keep_alive = None
         log.info(f"Websocket closed, cannot reconnect.")
         raise ConnectionClosed(self.cm, self.cm_list)
 
-    async def handle_logon(self, msg: MsgProto["CMsgClientLogonResponse"]) -> None:
+    async def handle_logon(self, msg: MsgProto[CMsgClientLogonResponse]) -> None:
         if msg.body.eresult == EResult.OK:
             log.debug("Logon completed")
 
             self.session_id = msg.session_id
-            self.cell_id = self.cm_list.cell_id = msg.body.cell_id
+            self.cm_list.cell_id = msg.body.cell_id
 
             interval = msg.body.out_of_game_heartbeat_seconds
             self._keep_alive = KeepAliveHandler(ws=self, interval=interval)
@@ -485,23 +499,28 @@ class SteamWebSocket:
             log.debug("Heartbeat started.")
 
             await self.send_um("ChatRoom.GetMyChatRoomGroups#1_Request")
-            status = MsgProto(EMsg.ClientChangeStatus, persona_state=EPersonaState.Online)
-            await self.send_as_proto(status)
-            # setting your status to offline will stop you receiving persona updates, don't ask why.
-            await self.send_as_proto(MsgProto(EMsg.ClientRequestCommentNotifications))
-        else:
-            if msg.body.eresult == EResult.InvalidPassword:
-                await asyncio.sleep(60)
-            await self.handle_close()
+            await self.change_presence(
+                games=self._connection._games,
+                state=self._connection._state,
+                ui_mode=self._connection._ui_mode,
+                flags=self._connection._flags,
+                force_kick=self._connection._force_kick,
+            )
+            return await self.send_as_proto(MsgProto(EMsg.ClientRequestCommentNotifications))
+        if msg.body.eresult == EResult.InvalidPassword:
+            http = self._connection.http
+            await http.logout()
+            await http.login(http.username, http.password, shared_secret=http.shared_secret)
+        await self.handle_close()
 
-    async def handle_multi(self, msg: MsgProto["CMsgMulti"]) -> None:
+    async def handle_multi(self, msg: MsgProto[CMsgMulti]) -> None:
         log.debug("Received a multi, unpacking")
         if msg.body.size_unzipped:
             log.debug(f"Decompressing payload ({len(msg.body.message_body)} -> {msg.body.size_unzipped})")
             # aiofiles is overrated :P
-            bytes_io = await self.loop.run_in_executor(EXECUTOR, BytesIO, msg.body.message_body)
-            gzipped = await self.loop.run_in_executor(EXECUTOR, functools.partial(GzipFile, fileobj=bytes_io))
-            data = await self.loop.run_in_executor(EXECUTOR, gzipped.read)
+            bytes_io = await utils.to_thread(BytesIO, msg.body.message_body)
+            gzipped = await utils.to_thread(GzipFile, fileobj=bytes_io)
+            data = await utils.to_thread(gzipped.read)
             if len(data) != msg.body.size_unzipped:
                 return log.info(f"Unzipped size mismatch for multi payload {msg}, discarding")
         else:
@@ -512,25 +531,39 @@ class SteamWebSocket:
             await self.receive(data[4 : 4 + size])
             data = data[4 + size :]
 
-    async def send_um(self, name: str, **kwargs) -> int:
-        msg = MsgProto(EMsg.ServiceMethodCallFromClient, um_name=name, **kwargs)
+    async def send_um(self, name: str, **kwargs: Any) -> int:
+        msg = MsgProto(EMsg.ServiceMethodCallFromClient, _um_name=name, **kwargs)
         msg.header.job_id_source = self._current_job_id = (self._current_job_id + 1) % 10000 or 1
         await self.send_as_proto(msg)
-        return self._current_job_id
+        return msg.header.job_id_source
+
+    async def send_um_and_wait(
+        self, name: str, check: Optional[Callable[[Msgs], bool]] = None, timeout: float = 5.0, **kwargs: Any
+    ) -> Msgs:
+        job_id = await self.send_um(name, **kwargs)
+        if check is None:
+            check = check_job_id(job_id)
+        return await asyncio.wait_for(self.wait_for(EMsg.ServiceMethodResponse, predicate=check), timeout=timeout)
 
     async def change_presence(
-        self, *, games: List[dict], state: Optional[EPersonaState], ui_mode: Optional["EUIMode"], force_kick: bool,
+        self,
+        *,
+        games: list[GameDict],
+        state: Optional[EPersonaState],
+        flags: int,
+        ui_mode: Optional[EUIMode],
+        force_kick: bool,
     ) -> None:
-        if force_kick is not None:
+        if force_kick:
             kick = MsgProto(EMsg.ClientKickPlayingSession)
             log.debug("Kicking any currently playing sessions")
             await self.send_as_proto(kick)
-        if games is not None:
+        if games:
             activity = MsgProto(EMsg.ClientGamesPlayedWithDataBlob, games_played=games)
             log.debug(f"Sending {activity} to change activity")
             await self.send_as_proto(activity)
-        if state is not None:
-            state = MsgProto(EMsg.ClientPersonaState, status_flags=state)
+        if state is not None or flags:
+            state = MsgProto(EMsg.ClientChangeStatus, persona_state=state, persona_state_flags=flags)
             log.debug(f"Sending {state} to change state")
             await self.send_as_proto(state)
         if ui_mode is not None:
