@@ -24,29 +24,34 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
+from __future__ import annotations
+
 import asyncio
+import html
 import json
 import logging
 import re
 from base64 import b64encode
 from sys import version_info
 from time import time
-from typing import TYPE_CHECKING, Any, Awaitable, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Coroutine, Optional
 
 import aiohttp
 import rsa
 from bs4 import BeautifulSoup
 
 from . import __version__, errors, utils
-from .models import URL
+from .models import URL, api_route, community_route
 from .user import ClientUser
 
 if TYPE_CHECKING:
     from .client import Client
     from .image import Image
+    from .user import User
 
 log = logging.getLogger(__name__)
-StrOrURL = Union[str, URL]
+StrOrURL = aiohttp.client.StrOrURL
+RequestType = Coroutine[None, None, Optional[Any]]
 
 
 async def json_or_text(r: aiohttp.ClientResponse) -> Optional[Any]:
@@ -56,22 +61,31 @@ async def json_or_text(r: aiohttp.ClientResponse) -> Optional[Any]:
         return await r.text()
 
 
-def api_route(path: str) -> URL:
-    return URL.API / f'{path}{"/v1" if path[-2:] != "v2" else ""}'
-
-
-def community_route(path: str) -> URL:
-    return URL.COMMUNITY / path
-
-
 class HTTPClient:
     """The HTTP Client that interacts with the Steam web API."""
 
+    __slots__ = (
+        "_session",
+        "_client",
+        "username",
+        "password",
+        "api_key",
+        "shared_secret",
+        "_one_time_code",
+        "_email_code",
+        "_captcha_id",
+        "_captcha_text",
+        "_steam_id",
+        "session_id",
+        "user",
+        "logged_in",
+        "user_agent",
+    )
+
     SUCCESS_LOG = "{method} {url} has received {text}"
     REQUEST_LOG = "{method} {url} with {payload} has returned {status}"
 
-    def __init__(self, loop: asyncio.AbstractEventLoop, client: "Client"):
-        self._loop = loop
+    def __init__(self, client: Client):
         self._session: Optional[aiohttp.ClientSession] = None  # filled in login
         self._client = client
 
@@ -80,6 +94,7 @@ class HTTPClient:
         self.api_key: Optional[str] = None
         self.shared_secret: Optional[str] = None
         self._one_time_code = ""
+        self._email_code = ""
         self._captcha_id = "-1"
         self._captcha_text = ""
         self._steam_id = ""
@@ -96,14 +111,14 @@ class HTTPClient:
         if self._session.closed:
             self._session = aiohttp.ClientSession()
 
-    async def request(self, method: str, url: StrOrURL, **kwargs) -> Optional[Any]:  # adapted from d.py
+    async def request(self, method: str, url: StrOrURL, **kwargs: Any) -> Optional[Any]:  # adapted from d.py
         kwargs["headers"] = {"User-Agent": self.user_agent, **kwargs.get("headers", {})}
 
         for tries in range(5):
             async with self._session.request(method, url, **kwargs) as r:
                 payload = kwargs.get("data")
                 log.debug(
-                    self.REQUEST_LOG.format(method=method, url=r.url, payload=f"PAYLOAD: {payload}", status=r.status,)
+                    self.REQUEST_LOG.format(method=method, url=r.url, payload=f"PAYLOAD: {payload}", status=r.status)
                 )
 
                 # even errors have text involved in them so this is safe to call
@@ -129,19 +144,19 @@ class HTTPClient:
                     await asyncio.sleep(1 + tries * 3)
                     continue
 
-                elif 300 <= r.status <= 399 and "login" in r.headers.get("location", ""):  # been logged out
+                # been logged out
+                elif 300 <= r.status <= 399 and "login" in r.headers.get("location", ""):
                     log.debug("Logged out of session re-logging in")
                     await self.login(self.username, self.password, self.shared_secret)
                     continue
 
                 elif r.status == 401:
-                    # api key either got revoked or it was never valid
                     if not data:
                         raise errors.HTTPException(r, data)
+                    # api key either got revoked or it was never valid
                     if "Access is denied. Retrying will not help. Please verify your <pre>key=</pre>" in data:
                         # time to fetch a new key
                         self._client.api_key = self.api_key = kwargs["key"] = await self.get_api_key()
-                        continue
                         # retry with our new key
 
                 # the usual error cases
@@ -155,9 +170,9 @@ class HTTPClient:
         # we've run out of retries, raise
         raise errors.HTTPException(r, data)
 
-    def connect_to_cm(self, cm: str) -> Awaitable:
+    def connect_to_cm(self, cm: str) -> Coroutine[None, None, aiohttp.ClientWebSocketResponse]:
         headers = {"User-Agent": self.user_agent}
-        return self._session.ws_connect(f"wss://{cm}/cmsocket/", timeout=60, headers=headers)
+        return self._session.ws_connect(f"wss://{cm}/cmsocket/", headers=headers)
 
     async def login(self, username: str, password: str, shared_secret: Optional[str]) -> None:
         self.username = username
@@ -167,10 +182,8 @@ class HTTPClient:
         self._session = aiohttp.ClientSession()
 
         resp = await self._send_login_request()
-        if "Please wait and try again later" in resp.get("message", ""):
-            raise errors.LoginError(resp["message"])
 
-        if resp.get("captcha_needed"):
+        if resp.get("captcha_needed") and resp.get("message") != "Please wait and try again later.":
             self._captcha_id = resp["captcha_gid"]
             print(
                 "Please enter the captcha text at"
@@ -197,41 +210,47 @@ class HTTPClient:
 
         resp = await self.get_user(resp["transfer_parameters"]["steamid"])
         data = resp["response"]["players"][0]
-        self.user = ClientUser(state=self._client._connection, data=data)
-        self._client._connection._users[self.user.id64] = self.user
+        state = self._client._connection
+        self.user = ClientUser(state=state, data=data)
+        state._users[self.user.id64] = self.user
         self.logged_in = True
         self._client.dispatch("login")
 
+    async def close(self) -> None:
+        await self.logout()
+        await self._session.close()
+
     async def logout(self) -> None:
         log.debug("Logging out of session")
         payload = {"sessionid": self.session_id}
         await self.request("POST", community_route("login/logout"), data=payload)
         self.logged_in = False
+        self.user = None
         self._client.dispatch("logout")
 
-    async def _get_rsa_params(self, current_repetitions: int = 0) -> Tuple[bytes, int]:
+    async def _get_rsa_params(self, current_repetitions: int = 0) -> tuple[bytes, int]:
         payload = {"username": self.username, "donotcache": int(time() * 1000)}
         try:
             key_response = await self.request("POST", community_route("login/getrsakey"), data=payload)
-        except Exception as e:
-            raise errors.LoginError from e
+        except Exception as exc:
+            raise errors.LoginError("Failed to get RSA key") from exc
         try:
-            rsa_mod = int(key_response["publickey_mod"], 16)
-            rsa_exp = int(key_response["publickey_exp"], 16)
+            n = int(key_response["publickey_mod"], 16)
+            e = int(key_response["publickey_exp"], 16)
             rsa_timestamp = key_response["timestamp"]
         except KeyError:
             if current_repetitions < 5:
                 return await self._get_rsa_params(current_repetitions + 1)
             raise ValueError("Could not obtain rsa-key")
         else:
-            return b64encode(rsa.encrypt(self.password.encode("utf-8"), rsa.PublicKey(rsa_mod, rsa_exp))), rsa_timestamp
+            return b64encode(rsa.encrypt(self.password.encode("utf-8"), rsa.PublicKey(n, e))), rsa_timestamp
 
     async def _send_login_request(self) -> dict:
         password, timestamp = await self._get_rsa_params()
         payload = {
             "username": self.username,
             "password": password.decode(),
-            "emailauth": self._one_time_code,
+            "emailauth": self._email_code,
             "emailsteamid": self._steam_id,
             "twofactorcode": self._one_time_code,
             "captchagid": self._captcha_id,
@@ -239,38 +258,37 @@ class HTTPClient:
             "loginfriendlyname": self.user_agent,
             "rsatimestamp": timestamp,
             "remember_login": True,
-            "donotcache": int(time() * 100000),
+            "donotcache": int(time() * 1000),
         }
         try:
             resp = await self.request("POST", community_route("login/dologin"), data=payload)
-            if resp.get("requires_twofactor") or resp.get("emailauth_needed"):
-                self._steam_id = resp.get("emailsteamid")
+            if resp.get("requires_twofactor"):
                 self._one_time_code = await self._client.code()
-                return await self._send_login_request()
-            return resp
+            elif resp.get("emailauth_needed"):
+                self._steam_id = resp.get("emailsteamid")
+                self._email_code = await self._client.code()
+            else:
+                return resp
+            return await self._send_login_request()
         except Exception as exc:
             raise errors.LoginError from exc
 
-    def get_user(self, user_id64: int) -> Awaitable:
+    def get_user(self, user_id64: int) -> RequestType:
         params = {"key": self.api_key, "steamids": user_id64}
         return self.request("GET", api_route("ISteamUser/GetPlayerSummaries/v2"), params=params)
 
-    async def get_users(self, user_id64s: List[int]) -> List[dict]:
+    async def get_users(self, user_id64s: list[int]) -> list[dict]:
         ret = []
+        if user_id64s == [0]:  # FIXME bandaid
+            return ret
 
-        def chunk():  # chunk the list into 100 element sub-lists for the requests
-            for i in range(0, len(user_id64s), 100):
-                yield user_id64s[i : i + 100]
-
-        for sublist in chunk():
-            for _ in sublist:
-                params = {"key": self.api_key, "steamids": ",".join(map(str, sublist))}
-
-            full_resp = await self.request("GET", api_route("ISteamUser/GetPlayerSummaries/v2"), params=params)
-            ret.extend(full_resp["response"]["players"])
+        for sublist in utils.chunk(user_id64s, 100):
+            params = {"key": self.api_key, "steamids": ",".join(map(str, sublist))}
+            resp = await self.request("GET", api_route("ISteamUser/GetPlayerSummaries/v2"), params=params)
+            ret.extend(resp["response"]["players"])
         return ret
 
-    def add_user(self, user_id64: int) -> Awaitable:
+    def add_user(self, user_id64: int) -> RequestType:
         payload = {
             "sessionID": self.session_id,
             "steamid": user_id64,
@@ -278,22 +296,22 @@ class HTTPClient:
         }
         return self.request("POST", community_route("actions/AddFriendAjax"), data=payload)
 
-    def remove_user(self, user_id64: int) -> Awaitable:
+    def remove_user(self, user_id64: int) -> RequestType:
         payload = {
             "sessionID": self.session_id,
             "steamid": user_id64,
         }
         return self.request("POST", community_route("actions/RemoveFriendAjax"), data=payload)
 
-    def block_user(self, user_id64: int) -> Awaitable:
+    def block_user(self, user_id64: int) -> RequestType:
         payload = {"sessionID": self.session_id, "steamid": user_id64, "block": 1}
         return self.request("POST", community_route("actions/BlockUserAjax"), data=payload)
 
-    def unblock_user(self, user_id64: int) -> Awaitable:
+    def unblock_user(self, user_id64: int) -> RequestType:
         payload = {"sessionID": self.session_id, "steamid": user_id64, "block": 0}
         return self.request("POST", community_route("actions/BlockUserAjax"), data=payload)
 
-    def accept_user_invite(self, user_id64: int) -> Awaitable:
+    def accept_user_invite(self, user_id64: int) -> RequestType:
         payload = {
             "sessionID": self.session_id,
             "steamid": user_id64,
@@ -301,7 +319,7 @@ class HTTPClient:
         }
         return self.request("POST", community_route("actions/AddFriendAjax"), data=payload)
 
-    def decline_user_invite(self, user_id64: int) -> Awaitable:
+    def decline_user_invite(self, user_id64: int) -> RequestType:
         payload = {
             "sessionID": self.session_id,
             "steamid": user_id64,
@@ -309,7 +327,7 @@ class HTTPClient:
         }
         return self.request("POST", community_route("actions/IgnoreFriendInviteAjax"), data=payload)
 
-    def get_user_games(self, user_id64: int) -> Awaitable:
+    def get_user_games(self, user_id64: int) -> RequestType:
         params = {
             "key": self.api_key,
             "steamid": user_id64,
@@ -318,13 +336,13 @@ class HTTPClient:
         }
         return self.request("GET", api_route("IPlayerService/GetOwnedGames"), params=params)
 
-    def get_user_inventory(self, user_id64: int, app_id: int, context_id: int) -> Awaitable:
+    def get_user_inventory(self, user_id64: int, app_id: int, context_id: int) -> RequestType:
         params = {
             "count": 5000,
         }
-        return self.request("GET", community_route(f"inventory/{user_id64}/{app_id}/{context_id}"), params=params,)
+        return self.request("GET", community_route(f"inventory/{user_id64}/{app_id}/{context_id}"), params=params)
 
-    def get_user_escrow(self, user_id64: int, token: Optional[str]) -> Awaitable:
+    def get_user_escrow(self, user_id64: int, token: Optional[str]) -> RequestType:
         params = {
             "key": self.api_key,
             "steamid_target": user_id64,
@@ -332,12 +350,12 @@ class HTTPClient:
         }
         return self.request("GET", api_route("IEconService/GetTradeHoldDurations"), params=params)
 
-    async def get_friends(self, user_id64: int) -> List[dict]:
+    async def get_friends(self, user_id64: int) -> list[dict]:
         params = {"key": self.api_key, "steamid": user_id64, "relationship": "friend"}
         friends = await self.request("GET", api_route("ISteamUser/GetFriendList"), params=params)
         return await self.get_users([friend["steamid"] for friend in friends["friendslist"]["friends"]])
 
-    def get_trade_offers(self, active_only: bool = True, sent: bool = True, received: bool = True) -> Awaitable:
+    def get_trade_offers(self, active_only: bool = True, sent: bool = True, received: bool = True) -> RequestType:
         params = {
             "key": self.api_key,
             "active_only": int(active_only),
@@ -347,7 +365,7 @@ class HTTPClient:
         }
         return self.request("GET", api_route("IEconService/GetTradeOffers"), params=params)
 
-    def get_trade_history(self, limit: int, previous_time: int) -> Awaitable:
+    def get_trade_history(self, limit: int, previous_time: Optional[int]) -> RequestType:
         params = {
             "key": self.api_key,
             "max_trades": limit,
@@ -357,11 +375,11 @@ class HTTPClient:
         }
         return self.request("GET", api_route("IEconService/GetTradeHistory"), params=params)
 
-    def get_trade(self, trade_id: int) -> Awaitable:
+    def get_trade(self, trade_id: int) -> RequestType:
         params = {"key": self.api_key, "tradeofferid": trade_id, "get_descriptions": 1}
         return self.request("GET", api_route("IEconService/GetTradeOffer"), params=params)
 
-    def accept_user_trade(self, user_id64: int, trade_id: int) -> Awaitable:
+    def accept_user_trade(self, user_id64: int, trade_id: int) -> RequestType:
         payload = {
             "sessionid": self.session_id,
             "tradeofferid": trade_id,
@@ -369,31 +387,30 @@ class HTTPClient:
             "partner": user_id64,
             "captcha": "",
         }
-        headers = {"Referer": f"{URL.COMMUNITY}/tradeoffer/{trade_id}"}
-        return self.request("POST", community_route(f"tradeoffer/{trade_id}/accept"), data=payload, headers=headers,)
+        headers = {"Referer": community_route(f"/tradeoffer/{trade_id}")}
+        return self.request("POST", community_route(f"tradeoffer/{trade_id}/accept"), data=payload, headers=headers)
 
-    def decline_user_trade(self, trade_id: int) -> Awaitable:
+    def decline_user_trade(self, trade_id: int) -> RequestType:
         payload = {"key": self.api_key, "tradeofferid": trade_id}
         return self.request("POST", api_route("IEconService/DeclineTradeOffer"), data=payload)
 
-    def cancel_user_trade(self, trade_id: int) -> Awaitable:
+    def cancel_user_trade(self, trade_id: int) -> RequestType:
         payload = {"key": self.api_key, "tradeofferid": trade_id}
         return self.request("POST", api_route("IEconService/CancelTradeOffer"), data=payload)
 
     def send_trade_offer(
         self,
-        user_id64: int,
-        user_id: int,
-        to_send: List[dict],
-        to_receive: List[dict],
+        user: User,
+        to_send: list[dict],
+        to_receive: list[dict],
         token: Optional[str],
         offer_message: str,
-        **kwargs,
-    ) -> Awaitable:
+        **kwargs: Any,
+    ) -> RequestType:
         payload = {
             "sessionid": self.session_id,
             "serverid": 1,
-            "partner": user_id64,
+            "partner": user.id64,
             "tradeoffermessage": offer_message,
             "json_tradeoffer": json.dumps(
                 {
@@ -407,52 +424,40 @@ class HTTPClient:
             "trade_offer_create_params": json.dumps({"trade_offer_access_token": token}) if token is not None else {},
         }
         payload.update(**kwargs)
-        headers = {"Referer": f"{URL.COMMUNITY}/tradeoffer/new/?partner={user_id}"}
+        headers = {"Referer": str(community_route(f"tradeoffer/new/?partner={user.id}"))}
         return self.request("POST", community_route("tradeoffer/new/send"), data=payload, headers=headers)
 
-    def send_counter_trade_offer(
-        self,
-        trade_id: int,
-        user_id64: int,
-        user_id: int,
-        to_send: List[dict],
-        to_receive: List[dict],
-        token: Optional[str],
-        offer_message: str,
-    ) -> Awaitable:
-        return self.send_trade_offer(user_id64, user_id, to_send, to_receive, token, offer_message, trade_id=trade_id,)
-
-    def get_cm_list(self, cell_id: int) -> Awaitable:
+    def get_cm_list(self, cell_id: int) -> RequestType:
         params = {"cellid": cell_id}
         return self.request("GET", api_route("ISteamDirectory/GetCMList"), params=params)
 
-    def get_comments(self, id64: int, comment_type: str, limit: Optional[int] = None) -> Awaitable:
+    def get_comments(self, id64: int, comment_path: str, limit: Optional[int] = None) -> RequestType:
         params = {"start": 0, "totalcount": 9999999999}
         if limit is None:
             params["count"] = 9999999999
         else:
             params["count"] = limit
-        return self.request("GET", community_route(f"comment/{comment_type}/render/{id64}"), params=params)
+        return self.request("GET", community_route(f"comment/{comment_path}/render/{id64}"), params=params)
 
-    def post_comment(self, id64: int, comment_type: str, content: str) -> Awaitable:
+    def post_comment(self, id64: int, comment_type: str, content: str) -> RequestType:
         payload = {
             "sessionid": self.session_id,
             "comment": content,
         }
         return self.request("POST", community_route(f"comment/{comment_type}/post/{id64}"), data=payload)
 
-    def delete_comment(self, id64: int, comment_id: int, comment_type: str) -> Awaitable:
+    def delete_comment(self, id64: int, comment_id: int, comment_type: str) -> RequestType:
         payload = {
             "sessionid": self.session_id,
             "gidcomment": comment_id,
         }
         return self.request("POST", community_route(f"comment/{comment_type}/delete/{id64}"), data=payload)
 
-    def report_comment(self, id64: int, comment_id: int, comment_type: str) -> Awaitable:
+    def report_comment(self, id64: int, comment_id: int, comment_type: str) -> RequestType:
         payload = {"gidcomment": comment_id, "hide": 1}
-        return self.request("POST", community_route(f"comment/{comment_type}/hideandreport/{id64}"), data=payload,)
+        return self.request("POST", community_route(f"comment/{comment_type}/hideandreport/{id64}"), data=payload)
 
-    def accept_clan_invite(self, clan_id: int) -> Awaitable:
+    def accept_clan_invite(self, clan_id: int) -> RequestType:
         payload = {
             "sessionid": self.session_id,
             "steamid": self.user.id64,
@@ -462,7 +467,7 @@ class HTTPClient:
         }
         return self.request("POST", community_route("my/friends/action"), data=payload)
 
-    def decline_clan_invite(self, clan_id: int) -> Awaitable:
+    def decline_clan_invite(self, clan_id: int) -> RequestType:
         payload = {
             "sessionid": self.session_id,
             "steamid": self.user.id64,
@@ -472,14 +477,14 @@ class HTTPClient:
         }
         return self.request("POST", community_route("my/friends/action"), data=payload)
 
-    def join_clan(self, clan_id: int) -> Awaitable:
+    def join_clan(self, clan_id: int) -> RequestType:
         payload = {
             "sessionID": self.session_id,
             "action": "join",
         }
         return self.request("POST", community_route(f"gid/{clan_id}"), data=payload)
 
-    def leave_clan(self, clan_id: int) -> Awaitable:
+    def leave_clan(self, clan_id: int) -> RequestType:
         payload = {
             "sessionID": self.session_id,
             "action": "leaveGroup",
@@ -487,7 +492,7 @@ class HTTPClient:
         }
         return self.request("POST", community_route("my/home_process"), data=payload)
 
-    def invite_user_to_clan(self, user_id64: int, clan_id: int) -> Awaitable:
+    def invite_user_to_clan(self, user_id64: int, clan_id: int) -> RequestType:
         payload = {
             "sessionID": self.session_id,
             "group": clan_id,
@@ -496,37 +501,57 @@ class HTTPClient:
         }
         return self.request("POST", community_route("actions/GroupInvite"), data=payload)
 
-    def get_user_clans(self, user_id64: int) -> Awaitable:
+    def get_user_clans(self, user_id64: int) -> RequestType:
         params = {"key": self.api_key, "steamid": user_id64}
         return self.request("GET", api_route("ISteamUser/GetUserGroupList"), params=params)
 
-    def get_user_bans(self, user_id64: int) -> Awaitable:
+    def get_user_bans(self, user_id64: int) -> RequestType:
         params = {"key": self.api_key, "steamids": user_id64}
         return self.request("GET", api_route("ISteamUser/GetPlayerBans"), params=params)
 
-    def get_user_level(self, user_id64: int) -> Awaitable:
+    def get_user_level(self, user_id64: int) -> RequestType:
         params = {"key": self.api_key, "steamid": user_id64}
         return self.request("GET", api_route("IPlayerService/GetSteamLevel"), params=params)
 
-    def get_user_badges(self, user_id64: int) -> Awaitable:
+    def get_user_badges(self, user_id64: int) -> RequestType:
         params = {"key": self.api_key, "steamid": user_id64}
         return self.request("GET", api_route("IPlayerService/GetBadges"), params=params)
 
-    def clear_nickname_history(self) -> Awaitable:
+    def clear_nickname_history(self) -> RequestType:
         payload = {"sessionid": self.session_id}
         return self.request("POST", community_route("my/ajaxclearaliashistory"), data=payload)
 
-    def clear_notifications(self) -> Awaitable:
+    def clear_notifications(self) -> RequestType:
         return self.request("GET", community_route("my/inventory"))
 
+    def get_price(self, app_id: int, item_name: str, currency: int) -> RequestType:
+        payload = {
+            "appid": app_id,
+            "market_hash_name": item_name,
+        }
+        payload.update({"currency": currency} if currency is not None else {})
+        return self.request("POST", community_route("market/priceoverview"), data=payload)
+
     async def edit_profile(
-        self, name: str, real_name: str, url: str, summary: str, country: str, state: str, city: str, avatar: "Image",
+        self,
+        name: Optional[str],
+        real_name: Optional[str],
+        url: Optional[str],
+        summary: Optional[str],
+        country: Optional[str],
+        state: Optional[str],
+        city: Optional[str],
+        avatar: Optional["Image"],
     ) -> None:
         if any((name, real_name, url, summary, country, state, city, avatar)):
             resp = await self.request("GET", url=community_route("my/edit"))
             soup = BeautifulSoup(resp, "html.parser")
             edit_config = str(soup.find("div", attrs={"id": "profile_edit_config"}))
-            value = re.findall(r'data-profile-edit=[\'"]{(.*?)},', utils.replace_html_code(edit_config), flags=re.S,)[0]
+            value = re.findall(
+                r'data-profile-edit=[\'"]{(.*?)},',
+                html.unescape(edit_config),
+                flags=re.S,
+            )[0]
             loadable = value.replace("\r", "\\r").replace("\n", "\\n")
             profile = json.loads(f'{"{"}{loadable}{"}}"}')
             for key, value in profile.items():
@@ -561,11 +586,11 @@ class HTTPClient:
             payload.add_field("sessionid", self.session_id)
             payload.add_field("doSub", "1")
             payload.add_field(
-                "avatar", avatar.read(), filename=f"avatar.{avatar.type}", content_type=f"image/{avatar.type}",
+                "avatar", avatar.read(), filename=f"avatar.{avatar.type}", content_type=f"image/{avatar.type}"
             )
             await self.request("POST", community_route("actions/FileUploader"), data=payload)
 
-    async def send_user_image(self, user_id64: int, image: "Image") -> None:
+    async def send_user_image(self, user_id64: int, image: Image) -> None:
         payload = {
             "sessionid": self.session_id,
             "l": "english",
@@ -595,7 +620,7 @@ class HTTPClient:
         )
         await self.request("POST", community_route("chat/commitfileupload"), data=payload)
 
-    async def send_group_image(self, destination: Tuple[int, int], image: "Image") -> None:
+    async def send_group_image(self, destination: tuple[int, int], image: Image) -> None:
         chat_id, channel_id = destination
         payload = {
             "sessionid": self.session_id,
