@@ -24,6 +24,8 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
+from __future__ import annotations
+
 import asyncio
 import gc
 import logging
@@ -33,24 +35,24 @@ from collections import deque
 from copy import copy
 from datetime import datetime
 from time import time
-from typing import TYPE_CHECKING, Awaitable, Callable, Dict, List, Mapping, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Callable, Coroutine, Deque, MutableMapping, Optional, Union
 
 from bs4 import BeautifulSoup
 from stringcase import snakecase
-from yarl import URL as _URL
+from typing_extensions import Protocol
 
 from . import utils
-from .abc import SteamID
+from .abc import SteamID, UserDict
 from .channel import ClanChannel, DMChannel, GroupChannel
 from .clan import Clan
-from .enums import EChatEntryType, EFriendRelationship, EResult, ETradeOfferState, EType
-from .errors import AuthenticatorError, InvalidCredentials, WSException, WSForbidden, WSNotFound
+from .enums import *
+from .errors import *
 from .group import Group
-from .guard import Confirmation, generate_confirmation_code, generate_device_id
+from .guard import *
 from .invite import ClanInvite, UserInvite
 from .message import *
 from .message import ClanMessage
-from .models import URL
+from .models import community_route
 from .protobufs import EMsg, MsgProto
 from .protobufs.steammessages_chat import (
     CChatRoomIncomingChatMessageNotification as GroupMessageNotification,
@@ -58,13 +60,16 @@ from .protobufs.steammessages_chat import (
 )
 from .protobufs.steammessages_friendmessages import (
     CFriendMessagesIncomingMessageNotification as UserMessageNotification,
+    CFriendMessagesSendMessageResponse as SendUserMessageResponse,
 )
-from .trade import TradeOffer
+from .trade import DescriptionDict, TradeOffer, TradeOfferDict
 from .user import User
 
 if TYPE_CHECKING:
+    from .abc import Message
     from .client import Client
     from .comment import Comment
+    from .gateway import SteamWebSocket
     from .http import HTTPClient
     from .protobufs.steammessages_chat import (
         CChatRoomChatRoomHeaderStateNotification as GroupStateUpdate,
@@ -81,30 +86,38 @@ if TYPE_CHECKING:
     from .protobufs.steammessages_clientserver_login import CMsgClientAccountInfo
 
 log = logging.getLogger(__name__)
-EventParser = Callable[["ConnectionState", "MsgProto"], Optional[Awaitable[None]]]
+
+
+class EventParser(Protocol):
+    def __call__(self, state: ConnectionState, msg: MsgProto) -> Optional[Coroutine[None, None, None]]:
+        ...
 
 
 class Registerer:
-    __slots__ = ("func", "emsg")
+    __slots__ = ("func", "emsg", "_state")
 
     def __init__(self, func: EventParser, emsg: EMsg):
         self.func = func
         self.emsg = emsg
+        self._state: ConnectionState
 
-    def __set_name__(self, state: "ConnectionState", _):
+    def __call__(self, *args: Any, **kwargs: Any) -> Optional[Coroutine[None, None, None]]:
+        return self.func(self._state, *args, **kwargs)
+
+    def __set_name__(self, state: ConnectionState, _) -> None:
         state.parsers[self.emsg] = self.func
+        self._state = state
 
 
 def register(emsg: EMsg) -> Callable[[EventParser], Registerer]:
-    def decorator(func: EventParser):
+    def decorator(func: EventParser) -> Registerer:
         return Registerer(func, emsg)
 
     return decorator
 
 
 class ConnectionState:
-    parsers: Dict[EMsg, EventParser] = dict()
-    # we need this outside __init__ for @register
+    parsers: dict[EMsg, EventParser] = dict()  # for @register
 
     __slots__ = (
         "loop",
@@ -114,7 +127,6 @@ class ConnectionState:
         "dispatch",
         "handled_friends",
         "handled_groups",
-        "user_slots",
         "invites",
         "max_messages",
         "_users",
@@ -122,7 +134,9 @@ class ConnectionState:
         "_groups",
         "_clans",
         "_confirmations",
+        "_confirmations_to_ignore",
         "_obj",
+        "_user_slots",
         "_previous_iteration",
         "_trades_task",
         "_trades_to_watch",
@@ -132,11 +146,14 @@ class ConnectionState:
         "_id64",
         "_device_id",
         "_messages",
+        "_games",
+        "_state",
+        "_ui_mode",
+        "_flags",
+        "_force_kick",
     )
 
-    def __init__(
-        self, loop: asyncio.AbstractEventLoop, client: "Client", http: "HTTPClient", **kwargs,
-    ):
+    def __init__(self, loop: asyncio.AbstractEventLoop, client: Client, http: HTTPClient, **kwargs: Any):
         self.loop = loop
         self.http = http
         self.request = http.request
@@ -144,25 +161,44 @@ class ConnectionState:
         self.dispatch = client.dispatch
 
         self.handled_friends = asyncio.Event()
-        self.user_slots = set(User.__slots__) - {"_state", "_data"}
-        self.max_messages = kwargs.pop("max_messages", 1000)
+        self._user_slots = set(User.__slots__) - {"_state"}
+        self.max_messages: int = kwargs.pop("max_messages", 1000)
+
+        game = kwargs.get("game")
+        games = kwargs.get("games")
+        games = [game.to_dict() for game in games] if games is not None else []
+        if game is not None:
+            games.append(game.to_dict())
+        self._games: list[dict] = games
+        self._state: EPersonaState = kwargs.get("state", EPersonaState.Online)
+        self._ui_mode: Optional[EUIMode] = kwargs.get("ui_mode")
+        flag: int = kwargs.get("flag")
+        flags: list[int] = kwargs.get("flag", [0])
+        if flag is not None:
+            flags.append(flag)
+        flag_value = 0
+        for flag in flags:
+            flag_value |= flag
+        self._flags: int = flag_value
+        self._force_kick: bool = kwargs.get("force_kick", False)
 
         self.clear()
 
     def clear(self) -> None:
-        self._users: Mapping[int, User] = weakref.WeakValueDictionary()
-        self._trades: Dict[int, TradeOffer] = dict()
-        self._groups: Dict[int, Group] = dict()
-        self._clans: Dict[int, Clan] = dict()
-        self._confirmations: Dict[int, Confirmation] = dict()
-        self._messages = self.max_messages and deque(maxlen=self.max_messages)
-        self.invites: List[Union[User, Clan]] = []
+        self._users: MutableMapping[int, User] = weakref.WeakValueDictionary()
+        self._trades: dict[int, TradeOffer] = dict()
+        self._groups: dict[int, Group] = dict()
+        self._clans: dict[int, Clan] = dict()
+        self._confirmations: dict[str, Confirmation] = dict()
+        self._confirmations_to_ignore: list[str] = []
+        self._messages: Deque[Message] = self.max_messages and deque(maxlen=self.max_messages)
+        self.invites: dict[int, Union[UserInvite, ClanInvite]] = dict()
 
         self._trades_task: Optional[asyncio.Task] = None
-        self._trades_to_watch: List[int] = []
-        self._trades_received_cache: List[dict] = []
-        self._trades_sent_cache: List[dict] = []
-        self._descriptions_cache: List[dict] = []
+        self._trades_to_watch: list[int] = []
+        self._trades_received_cache: list[dict] = []
+        self._trades_sent_cache: list[dict] = []
+        self._descriptions_cache: list[dict] = []
 
         self.handled_friends.clear()
         self.handled_groups = False
@@ -178,23 +214,27 @@ class ConnectionState:
         await self._poll_trades()
 
     @property
-    def users(self) -> List[User]:
+    def ws(self) -> SteamWebSocket:
+        return self.client.ws
+
+    @property
+    def users(self) -> list[User]:
         return list(self._users.values())
 
     @property
-    def trades(self) -> List[TradeOffer]:
+    def trades(self) -> list[TradeOffer]:
         return list(self._trades.values())
 
     @property
-    def groups(self) -> List[Group]:
+    def groups(self) -> list[Group]:
         return list(self._groups.values())
 
     @property
-    def clans(self) -> List[Clan]:
+    def clans(self) -> list[Clan]:
         return list(self._clans.values())
 
     @property
-    def confirmations(self) -> List[Confirmation]:
+    def confirmations(self) -> list[Confirmation]:
         return list(self._confirmations.values())
 
     def get_user(self, id64: int) -> Optional[User]:
@@ -205,11 +245,11 @@ class ConnectionState:
         players = resp["response"]["players"]
         return User(state=self, data=players[0]) if players else None
 
-    async def fetch_users(self, user_id64s: List[int]) -> List[Optional[User]]:
+    async def fetch_users(self, user_id64s: list[int]) -> list[Optional[User]]:
         resp = await self.http.get_users(user_id64s)
         return [User(state=self, data=data) for data in resp]
 
-    def _store_user(self, data: dict) -> User:
+    def _store_user(self, data: UserDict) -> User:
         try:
             user = self._users[int(data["steamid"])]
         except KeyError:
@@ -218,26 +258,23 @@ class ConnectionState:
         return user
 
     def get_confirmation(self, id: int) -> Optional[Confirmation]:
-        return self._confirmations.get(id)
+        confirmation = [c for c in self._confirmations.values() if c.trade_id == id]
+        return confirmation[0] if confirmation else None
 
     async def fetch_confirmation(self, id: int) -> Optional[Confirmation]:
         await self._fetch_confirmations()
-        return self._confirmations.get(id)
+        return self.get_confirmation(id)
 
-    def get_group(self, id: int) -> Optional["Group"]:
+    def get_group(self, id: int) -> Optional[Group]:
         return self._groups.get(id)
 
-    def get_clan(self, id: int) -> Optional["Clan"]:
+    def get_clan(self, id: int) -> Optional[Clan]:
         return self._clans.get(id)
 
-    async def fetch_clan(self, id64: int) -> Optional["Clan"]:
-        job_id = await self.client.ws.send_um(
-            "ClanChatRooms.GetClanChatRoomInfo#1_Request", steamid=id64, autocreate=True
-        )
+    async def fetch_clan(self, id64: int) -> Optional[Clan]:
         try:
-            msg: MsgProto["FetchGroupResponse"] = await asyncio.wait_for(
-                self.client.ws.wait_for(EMsg.ServiceMethodResponse, lambda m: m.header.job_id_target == job_id,),
-                timeout=5,
+            msg: MsgProto[FetchGroupResponse] = await self.ws.send_um_and_wait(
+                "ClanChatRooms.GetClanChatRoomInfo#1_Request", steamid=id64, autocreate=True
             )
         except asyncio.TimeoutError:
             return None
@@ -260,14 +297,17 @@ class ConnectionState:
             return trades[0]
         return None
 
-    async def _store_trade(self, data: dict) -> TradeOffer:
+    async def _store_trade(self, data: TradeOfferDict) -> TradeOffer:
         try:
             trade = self._trades[int(data["tradeofferid"])]
         except KeyError:
             log.info(f'Received trade #{data["tradeofferid"]}')
             trade = await TradeOffer._from_api(state=self, data=data)
             self._trades[trade.id] = trade
-            if trade.state not in (ETradeOfferState.Active, ETradeOfferState.ConfirmationNeed,):
+            if trade.state not in (
+                ETradeOfferState.Active,
+                ETradeOfferState.ConfirmationNeed,
+            ):
                 return trade
             if trade.is_our_offer():
                 self.dispatch("trade_send", trade)
@@ -275,20 +315,19 @@ class ConnectionState:
                 self.dispatch("trade_receive", trade)
             self._trades_to_watch.append(trade.id)
         else:
-            before_state = copy(trade.state)
+            before_state = trade.state
             trade._update(data)
-            if data["trade_offer_state"] != before_state:
+            if trade.state != before_state:
                 log.info(f"Trade #{trade.id} has updated its trade state to {trade.state}")
-                states = {
-                    ETradeOfferState.Accepted: "accept",
-                    ETradeOfferState.Countered: "counter",
-                    ETradeOfferState.Expired: "expire",
-                    ETradeOfferState.Canceled: "cancel",
-                    ETradeOfferState.Declined: "decline",
-                    ETradeOfferState.CanceledBySecondaryFactor: "cancel",
-                }
                 try:
-                    event_name = states[trade.state]
+                    event_name = {
+                        ETradeOfferState.Accepted: "accept",
+                        ETradeOfferState.Countered: "counter",
+                        ETradeOfferState.Expired: "expire",
+                        ETradeOfferState.Canceled: "cancel",
+                        ETradeOfferState.Declined: "decline",
+                        ETradeOfferState.CanceledBySecondaryFactor: "cancel",
+                    }[trade.state]
                 except KeyError:
                     pass
                 else:
@@ -296,7 +335,9 @@ class ConnectionState:
                     self._trades_to_watch.remove(trade.id)
         return trade
 
-    async def _process_trades(self, trades: List[dict], descriptions: List[dict]) -> List[TradeOffer]:
+    async def _process_trades(
+        self, trades: list[TradeOfferDict], descriptions: list[DescriptionDict]
+    ) -> list[TradeOffer]:
         ret = []
         for trade in trades:
             for item in descriptions:
@@ -325,23 +366,22 @@ class ConnectionState:
         self._trades_sent_cache = trades_sent
         self._descriptions_cache = descriptions
 
-    async def _parse_comment(self) -> Optional["Comment"]:
-        # this isn't very efficient but I'm not sure if it can be done better
-        resp = await self.request("GET", f"{URL.COMMUNITY}/my/commentnotifications")
+    async def _parse_comment(self) -> Optional[Comment]:
+        resp = await self.request("GET", community_route("my/commentnotifications"))
         search = re.search(r'<div class="commentnotification_click_overlay">\s*<a href="(.*?)">', resp)
         if search is None:
-            return None
-        steam_id = await SteamID.from_url(f"{URL.COMMUNITY}{_URL(search.group(1)).path}", self.http._session)
+            return
+        steam_id = await SteamID.from_url(search.group(1), self.http._session)
         if steam_id is None:
-            return None
+            return
         if steam_id.type == EType.Clan:
             obj = await self.fetch_clan(steam_id.id64)
         else:
             obj = await self.fetch_user(steam_id.id64)
         if obj is None:
-            return None
+            return
 
-        if self._obj == obj:
+        if self._obj is obj:
             self._previous_iteration += 1
         else:
             self._obj = obj
@@ -350,7 +390,7 @@ class ConnectionState:
         try:
             return comments[self._previous_iteration]
         except KeyError:
-            return None
+            pass
 
     # confirmations
 
@@ -365,10 +405,10 @@ class ConnectionState:
             "tag": tag,
         }
 
-    async def _fetch_confirmations(self) -> Optional[dict]:
+    async def _fetch_confirmations(self) -> dict[str, Confirmation]:
         params = self._create_confirmation_params("conf")
         headers = {"X-Requested-With": "com.valvesoftware.android.steam.community"}
-        resp = await self.request("GET", f"{URL.COMMUNITY}/mobileconf/conf", params=params, headers=headers)
+        resp = await self.request("GET", community_route("mobileconf/conf"), params=params, headers=headers)
 
         if "incorrect Steam Guard codes." in resp:
             raise InvalidCredentials("identity_secret is incorrect")
@@ -377,13 +417,21 @@ class ConnectionState:
 
         soup = BeautifulSoup(resp, "html.parser")
         if soup.select("#mobileconf_empty"):
-            return None
+            return {}
         for confirmation in soup.select("#mobileconf_list .mobileconf_list_entry"):
             id = confirmation["id"]
             confid = confirmation["data-confid"]
             key = confirmation["data-key"]
             trade_id = int(confirmation.get("data-creator", 0))
-            self._confirmations[trade_id] = Confirmation(self, id, confid, key, trade_id)
+            confirmation_id = id.split("conf")[1]
+            if confirmation_id in self._confirmations_to_ignore:
+                continue
+            self._confirmations[confirmation_id] = Confirmation(
+                self, confirmation_id, confid, key, trade_id, f"details{confid}"
+            )
+        for confirmation_id in self._confirmations_to_ignore:
+            if confirmation_id in self._confirmations:
+                del self._confirmations[confirmation_id]
         return self._confirmations
 
     def _generate_confirmation(self, tag: str, timestamp: int) -> str:
@@ -401,24 +449,20 @@ class ConnectionState:
     # ws stuff
 
     @property
-    def _combined(self) -> Dict[int, Union["Group", "Clan"]]:
+    def _combined(self) -> dict[int, Union[Group, Clan]]:
         return {
-            **{group.id: group for group in self.groups},
+            **self._groups,
             **{clan.chat_id: clan for clan in self.clans},
         }
 
     async def send_user_message(self, user_id64: int, content: str) -> None:
-        job_id = await self.client.ws.send_um(
-            "FriendMessages.SendMessage#1_Request",
-            steamid=str(user_id64),
-            message=content,
-            chat_entry_type=EChatEntryType.Text,
-            contains_bbcode=utils.contains_bbcode(content),
-        )
         try:
-            msg = await asyncio.wait_for(
-                self.client.ws.wait_for(EMsg.ServiceMethodResponse, lambda m: m.header.job_id_target == job_id,),
-                timeout=5,
+            msg: MsgProto[SendUserMessageResponse] = await self.ws.send_um_and_wait(
+                "FriendMessages.SendMessage#1_Request",
+                steamid=str(user_id64),
+                message=content,
+                chat_entry_type=EChatEntryType.Text,
+                contains_bbcode=utils.contains_bbcode(content),
             )
         except asyncio.TimeoutError:
             return
@@ -432,6 +476,7 @@ class ConnectionState:
             chat_entry_type=EChatEntryType.Text,
             message=content,
             rtime32_server_timestamp=int(time()),
+            message_no_bbcode=msg.body.message_without_bb_code,
         )
         channel = DMChannel(state=self, participant=self.get_user(user_id64))
         message = UserMessage(proto=proto, channel=channel)
@@ -439,21 +484,19 @@ class ConnectionState:
         self._messages.append(message)
         self.dispatch("message", message)
 
-    async def send_user_typing(self, user: "User") -> None:
-        await self.client.ws.send_um(
-            "FriendMessages.SendMessage#1_Request", steamid=str(user.id64), chat_entry_type=EChatEntryType.Typing,
+    async def send_user_typing(self, user: User) -> None:
+        await self.ws.send_um(
+            "FriendMessages.SendMessage#1_Request",
+            steamid=str(user.id64),
+            chat_entry_type=EChatEntryType.Typing,
         )
         self.dispatch("typing", self.client.user, datetime.utcnow())
 
-    async def send_group_message(self, destination: Tuple[int, int], content: str) -> None:
+    async def send_group_message(self, destination: tuple[int, int], content: str) -> None:
         chat_id, group_id = destination
-        job_id = await self.client.ws.send_um(
-            "ChatRoom.SendChatMessage#1_Request", chat_id=chat_id, chat_group_id=group_id, message=content,
-        )
         try:
-            msg = await asyncio.wait_for(
-                self.client.ws.wait_for(EMsg.ServiceMethodResponse, lambda m: m.header.job_id_target == job_id,),
-                timeout=5,
+            msg = await self.ws.send_um_and_wait(
+                "ChatRoom.SendChatMessage#1_Request", chat_id=chat_id, chat_group_id=group_id, message=content
             )
         except asyncio.TimeoutError:
             return
@@ -465,7 +508,7 @@ class ConnectionState:
             raise WSException(msg)
 
         proto = GroupMessageNotification(
-            chat_id=chat_id, chat_group_id=group_id, steamid_sender=0, message=content, timestamp=int(time()),
+            chat_id=chat_id, chat_group_id=group_id, steamid_sender=0, message=content, timestamp=int(time())
         )
         destination = self._combined.get(group_id)
         if isinstance(destination, Clan):
@@ -478,13 +521,9 @@ class ConnectionState:
         self.dispatch("message", message)
 
     async def join_chat(self, chat_id: int, invite_code: Optional[str] = None) -> None:
-        job_id = await self.client.ws.send_um(
-            "ChatRoom.JoinChatRoomGroup#1_Request", chat_group_id=chat_id, invite_code=invite_code or "",
-        )
         try:
-            msg = await asyncio.wait_for(
-                self.client.ws.wait_for(EMsg.ServiceMethodResponse, lambda m: m.header.job_id_target == job_id,),
-                timeout=5,
+            msg = await self.ws.send_um_and_wait(
+                "ChatRoom.JoinChatRoomGroup#1_Request", chat_group_id=chat_id, invite_code=invite_code or ""
             )
         except asyncio.TimeoutError:
             return
@@ -494,11 +533,31 @@ class ConnectionState:
             raise WSException(msg)
 
     async def leave_chat(self, chat_id: int) -> None:
-        job_id = await self.client.ws.send_um("ChatRoom.LeaveChatRoomGroup#1_Request", chat_group_id=chat_id)
         try:
-            msg = await asyncio.wait_for(
-                self.client.ws.wait_for(EMsg.ServiceMethodResponse, lambda m: m.header.job_id_target == job_id,),
-                timeout=5,
+            msg = await self.ws.send_um_and_wait("ChatRoom.LeaveChatRoomGroup#1_Request", chat_group_id=chat_id)
+        except asyncio.TimeoutError:
+            return
+        if msg.header.eresult == EResult.InvalidParameter:
+            raise WSNotFound(msg)
+        elif msg.header.eresult != EResult.OK:
+            raise WSException(msg)
+
+    async def invite_user_to_group(self, user_id64: int, group_id: int) -> None:
+        try:
+            msg = await self.ws.send_um_and_wait(
+                "InviteFriendToChatRoomGroup#1_Request", chat_group_id=group_id, steamid=user_id64
+            )
+        except asyncio.TimeoutError:
+            return
+        if msg.header.eresult == EResult.InvalidParameter:
+            raise WSNotFound(msg)
+        elif msg.header.eresult != EResult.OK:
+            raise WSException(msg)
+
+    async def edit_role(self, group_id: int, role_id: int, *, name: str) -> None:
+        try:
+            msg = await self.ws.send_um_and_wait(
+                "ChatRoom.RenameRole#1_Request", chat_group_id=group_id, role_id=role_id, name=name
             )
         except asyncio.TimeoutError:
             return
@@ -512,7 +571,7 @@ class ConnectionState:
     @register(EMsg.ServiceMethod)
     async def parse_service_method(self, msg: MsgProto) -> None:
         if msg.header.job_name_target == "FriendMessagesClient.IncomingMessage#1":
-            msg: MsgProto["UserMessageNotification"]
+            msg: MsgProto[UserMessageNotification]
             user_id64 = msg.body.steamid_friend
             author = self.get_user(user_id64) or await self.fetch_user(user_id64)
             if author is None:
@@ -529,7 +588,7 @@ class ConnectionState:
                 self.dispatch("typing", author, when)
 
         if msg.header.job_name_target == "ChatRoomClient.NotifyIncomingChatMessage#1":
-            msg: MsgProto["GroupMessageNotification"]
+            msg: MsgProto[GroupMessageNotification]
             destination = self._combined.get(msg.body.chat_group_id)
             if destination is None:
                 return
@@ -547,7 +606,7 @@ class ConnectionState:
             self.dispatch("message", message)
 
         if msg.header.job_name_target == "ChatRoomClient.NotifyChatRoomHeaderStateChange#1":  # group update
-            msg: MsgProto["GroupStateUpdate"]
+            msg: MsgProto[GroupStateUpdate]
             destination = self._combined.get(msg.body.header_state.chat_group_id)
             if destination is None:
                 return
@@ -558,12 +617,12 @@ class ConnectionState:
                 destination._from_proto(msg.body.header_state)
 
         if msg.header.job_name_target == "ChatRoomClient.NotifyChatGroupUserStateChanged#1":
-            msg: MsgProto["GroupAction"]
+            msg: MsgProto[GroupAction]
             if msg.body.user_action == "Joined":  # join group
                 if msg.body.group_summary.clanid:
                     clan = await Clan._from_proto(self, msg.body.group_summary)
                     self._clans[clan.id] = clan
-                    self.dispatch("clan_join", clan)
+                    self.dispatch("clan_join", clan)  # TODO test/doc
                 else:
                     group = Group(state=self, proto=msg.body.group_summary)
                     self._groups[group.id] = group
@@ -575,14 +634,14 @@ class ConnectionState:
                     return
 
                 if isinstance(left, Clan):
-                    self.dispatch("clan_leave", left)
+                    self.dispatch("clan_leave", left)  # TODO test/doc
                 else:
                     self.dispatch("group_leave", left)
 
     @register(EMsg.ServiceMethodResponse)
     async def parse_service_method_response(self, msg: MsgProto) -> None:
         if msg.header.job_name_target == "ChatRoom.GetMyChatRoomGroups#1":
-            msg: MsgProto["MyChatRooms"]
+            msg: MsgProto[MyChatRooms]
             for group in msg.body.chat_room_groups:
                 if group.group_summary.clanid:  # received a clan
                     clan = await Clan._from_proto(self, group)
@@ -597,50 +656,50 @@ class ConnectionState:
                 self.client._handle_ready()
 
     @register(EMsg.ClientCMList)
-    def parse_cm_list_update(self, msg: MsgProto["CMsgClientCMList"]) -> None:
+    def parse_cm_list_update(self, msg: MsgProto[CMsgClientCMList]) -> None:
         log.debug("Updating CM list")
         cms = msg.body.cm_websocket_addresses
-        self.client.ws.cm_list.clear()
-        self.client.ws.cm_list.merge_list(cms)
-        self.loop.create_task(self.client.ws.cm_list.ping_cms(to_ping=len(cms)))
+        self.ws.cm_list.clear()
+        self.ws.cm_list.merge_list(cms)
+        self.loop.create_task(self.ws.cm_list.ping_cms(to_ping=len(cms)))
         # ping all the cms, we have time.
 
     @register(EMsg.ClientPersonaState)
-    async def parse_persona_state_update(self, msg: MsgProto["CMsgClientPersonaState"]) -> None:
+    async def parse_persona_state_update(self, msg: MsgProto[CMsgClientPersonaState]) -> None:
         for friend in msg.body.friends:
-            data = friend.to_dict(snakecase)
+            data: UserDict = friend.to_dict(snakecase)
             if not data:
                 continue
             user_id64 = friend.friendid
             after = self.get_user(user_id64)
-            before = copy(after)
             if after is None:  # they're private
                 continue
+            before = copy(after)
 
             try:
                 data = self.patch_user_from_ws(data, friend)
             except (KeyError, TypeError):
                 steam_id = SteamID(user_id64)
                 invitee = await self.fetch_user(steam_id.id64) or steam_id
-                invite = UserInvite(self, invitee)
+                invite = UserInvite(self, invitee, None)
                 self.dispatch("user_invite", invite)
 
             after._update(data)
-            old = [getattr(before, attr) for attr in self.user_slots]
-            new = [getattr(after, attr) for attr in self.user_slots]
+            old = [getattr(before, attr) for attr in self._user_slots]
+            new = [getattr(after, attr) for attr in self._user_slots]
             if old != new:
                 self.dispatch("user_update", before, after)
 
-    def patch_user_from_ws(self, data: dict, friend: "CMsgClientPersonaStateFriend") -> dict:
+    def patch_user_from_ws(self, data: dict, friend: CMsgClientPersonaStateFriend) -> dict:
         data["personaname"] = friend.player_name
         hash = (
             friend.avatar_hash.hex()
             if data["avatar_hash"] != "\x00" * 20
             else "fef49e7fa7e1997310d705b2a6158ff8dc1cdfeb"
         )
-        data["avatarfull"] = (
-            f"https://steamcdn-a.akamaihd.net/steamcommunity/public/images/avatars/{hash[:2]}/{hash}_full.jpg"
-        )
+        data[
+            "avatarfull"
+        ] = f"https://steamcdn-a.akamaihd.net/steamcommunity/public/images/avatars/{hash[:2]}/{hash}_full.jpg"
 
         if friend.last_logoff:
             data["lastlogoff"] = friend.last_logoff
@@ -650,7 +709,7 @@ class ConnectionState:
         return data
 
     @register(EMsg.ClientFriendsList)
-    async def process_friends(self, msg: MsgProto["CMsgClientFriendsList"]) -> None:
+    async def process_friends(self, msg: MsgProto[CMsgClientFriendsList]) -> None:
         if not self.handled_friends.is_set():
             self.client.user.friends = await self.fetch_users(
                 [
@@ -673,13 +732,14 @@ class ConnectionState:
                 EFriendRelationship.RequestRecipient,
             ):
                 steam_id = SteamID(user.ulfriendid)
+                relationship = EFriendRelationship(user.efriendrelationship)
                 if steam_id.type == EType.Individual:
                     invitee = await self.fetch_user(steam_id.id64) or steam_id
-                    invite = UserInvite(self, invitee)
-                    self.invites.append(invitee)
+                    invite = UserInvite(state=self, invitee=invitee, relationship=relationship)
+                    self.invites[invitee.id64] = invite
                     self.dispatch("user_invite", invite)
                 if steam_id.type == EType.Clan:
-                    resp = await self.request("GET", f"{URL.COMMUNITY}/my/groups/pending?ajax=1")
+                    resp = await self.request("GET", community_route("my/groups/pending?ajax=1"))
                     soup = BeautifulSoup(resp, "html.parser")
                     elements = soup.find_all("a", attrs={"class": "linkStandard"})
                     invitee_id = 0
@@ -689,29 +749,24 @@ class ConnectionState:
                             break
                     invitee = self.get_user(invitee_id) or await self.fetch_user(invitee_id) or SteamID(invitee_id)
                     clan = self.get_clan(steam_id.id64) or await self.client.fetch_clan(steam_id.id64) or steam_id
-                    invite = ClanInvite(state=self, invitee=invitee, clan=clan)
+                    invite = ClanInvite(state=self, invitee=invitee, clan=clan, relationship=relationship)
                     self.dispatch("clan_invite", invite)
-                    self.invites.append(clan)
+                    self.invites[clan.id64] = invite
 
-            # TODO
-            """ 
-            async def fetch_invites() -> None:  # we dont want to spend ages waiting for invites
-                if user.efriendrelationship == EFriendRelationship.Friend:
-                    steam_id = SteamID(user.ulfriendid)
+            if user.efriendrelationship == EFriendRelationship.Friend:
+                steam_id = SteamID(user.ulfriendid)
+                try:
+                    invite = self.invites.pop(steam_id.id64)
+                except KeyError:
+                    pass
+                else:
                     if steam_id.type == EType.Individual:
-                        invitee = self.get_user(steam_id.id64) or await self.fetch_user(steam_id.id64) or steam_id
-                        if invitee in self.invites:
-                            self.dispatch('user_invite_accept', invitee)
-                    if steam_id.type == EType.Clan:
-                        clan = self.get_clan(steam_id.id) or await self.client.fetch_clan(steam_id.id64) or steam_id
-                        if clan in self.invites:
-                            self.dispatch('clan_invite_accept', clan)
-
-            self.loop.create_task(fetch_invites())
-            """
+                        self.dispatch("user_invite_accept", invite)
+                    else:
+                        self.dispatch("clan_invite_accept", invite)
 
     @register(EMsg.ClientCommentNotifications)
-    async def handle_comments(self, msg: MsgProto["CMsgClientCommentNotifications"]) -> None:
+    async def handle_comments(self, msg: MsgProto[CMsgClientCommentNotifications]) -> None:
         for _ in range(msg.body.count_new_comments):
             comment = await self._parse_comment()
             if comment is not None:
@@ -720,7 +775,7 @@ class ConnectionState:
         await self.http.clear_notifications()
 
     @register(EMsg.ClientUserNotifications)
-    async def parse_notification(self, msg: MsgProto["CMsgClientUserNotifications"]) -> None:
+    async def parse_notification(self, msg: MsgProto) -> None:
         for notification in msg.body.notifications:
             if notification.type == 1:  # received a trade offer
 
@@ -739,7 +794,7 @@ class ConnectionState:
             await self.http.clear_notifications()
 
     @register(EMsg.ClientAccountInfo)
-    def parse_account_info(self, msg: MsgProto["CMsgClientAccountInfo"]):
+    def parse_account_info(self, msg: MsgProto[CMsgClientAccountInfo]) -> None:
         if msg.body.persona_name != self.client.user.name:
             before = copy(self.client.user)
             self.client.user.name = msg.body.persona_name
