@@ -24,9 +24,12 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
-import asyncio
+from __future__ import annotations
+
 from datetime import datetime
-from typing import TYPE_CHECKING, Iterable, List, Optional, Union
+from typing import TYPE_CHECKING, Any, Generator, Optional, Union
+
+from typing_extensions import TypedDict
 
 from .enums import ETradeOfferState
 from .errors import ClientException, ConfirmationError
@@ -44,9 +47,76 @@ __all__ = (
     "TradeOffer",
 )
 
+Items = Union["Item", "Asset"]
+
+
+# TypedDicts to help visualise the data you receive
+class AssetToDict(TypedDict):
+    assetid: str
+    amount: int
+    appid: str
+    contextid: str
+
+
+class AssetDict(AssetToDict):
+    instanceid: str
+    classid: str
+    missing: bool
+
+
+class DescriptionDict(TypedDict, total=False):
+    instanceid: str
+    classid: str
+    market_name: str
+    currency: int
+    name: str
+    market_hash_name: str
+    name_color: str
+    background_color: str  # hex code
+    type: str
+    descriptions: dict[str, str]
+    market_actions: list[dict[str, str]]
+    tags: list[dict[str, str]]
+    actions: list[dict[str, str]]
+    icon_url: str
+    icon_url_large: str
+    tradable: bool  # 1 vs 0
+    marketable: bool  # same as above
+    commodity: int  # might be a bool
+
+
+class ItemDict(AssetDict, DescriptionDict):
+    """We combine Assets with their matching Description to form items."""
+
+
+class InventoryDict(TypedDict):
+    assets: list[AssetDict]
+    descriptions: list[DescriptionDict]
+    total_inventory_count: int
+    success: int  # EResult
+    rwgrsn: int  # p. much always -2
+
+
+class TradeOfferDict(TypedDict):
+    tradeofferid: str
+    tradeid: str  # no clue what this is (its not the useful one)
+    accountid_other: int
+    message: str
+    trade_offer_state: int  # ETradeOfferState
+    expiration_time: int  # unix timestamps
+    time_created: int
+    time_updated: int
+    escrow_end_date: int
+    items_to_give: list[ItemDict]
+    items_to_receive: list[ItemDict]
+    is_our_offer: bool
+    from_real_time_trade: bool
+    confirmation_method: int  # 2 is mobile not a clue what other values are
+
 
 class Asset:
-    """A striped down version of an item.
+    """Base most version of an item. This class should only be received when Steam fails to find a matching item for
+    its class and instance IDs.
 
     .. container:: operations
 
@@ -64,8 +134,6 @@ class Asset:
         The game the item is from.
     asset_id: :class:`str`
         The assetid of the item.
-    app_id: :class:`str`
-        The appid of the item.
     amount: :class:`int`
         The amount of the same asset there are in the inventory.
     instance_id: :class:`str`
@@ -74,17 +142,17 @@ class Asset:
         The classid of the item.
     """
 
-    __slots__ = ("game", "amount", "app_id", "class_id", "asset_id", "instance_id")
+    __slots__ = ("game", "amount", "class_id", "asset_id", "instance_id", "name")
 
-    def __init__(self, data: dict):
+    def __init__(self, data: AssetDict):
         self.asset_id = int(data["assetid"])
-        self.game = Game(app_id=data["appid"])
-        self.app_id = self.game.app_id
+        self.game = Game(id=data["appid"])
         self.amount = int(data["amount"])
         self.instance_id = int(data["instanceid"])
         self.class_id = int(data["classid"])
+        self.name = None  # so we don't raise AttributeError
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         attrs = (
             "game",
             "amount",
@@ -95,14 +163,14 @@ class Asset:
         resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
         return f"<Asset {' '.join(resolved)}>"
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         return isinstance(other, Asset) and self.instance_id == other.instance_id and self.class_id == other.class_id
 
-    def to_dict(self) -> dict:
+    def to_dict(self) -> AssetToDict:
         return {
             "assetid": str(self.asset_id),
             "amount": self.amount,
-            "appid": str(self.app_id),
+            "appid": str(self.game.id),
             "contextid": str(self.game.context_id),
         }
 
@@ -147,24 +215,22 @@ class Item(Asset):
         "icon_url",
         "display_name",
         "descriptions",
-        "_state",
         "_is_tradable",
         "_is_marketable",
     )
 
-    def __init__(self, state: "ConnectionState", data: dict):
+    def __init__(self, data: ItemDict):
         super().__init__(data)
-        self._state = state
         self._from_data(data)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         asset_repr = super().__repr__()[7:-1]
         attrs = ("name",)
         resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
         resolved.append(asset_repr)
         return f"<Item {' '.join(resolved)}>"
 
-    def _from_data(self, data) -> None:
+    def _from_data(self, data: ItemDict) -> None:
         self.name = data.get("market_name")
         self.display_name = data.get("name")
         self.colour = int(data["name_color"], 16) if "name_color" in data else None
@@ -203,48 +269,47 @@ class Inventory:
 
         .. describe:: y in x
 
-            Determines if an item is in the inventory based off of
-            its class_id and instance_id
+            Determines if an item is in the inventory based off of its class_id and instance_id.
 
 
     Attributes
     -------------
-    items: List[Union[:class:`Item`, :class:`Asset`]
+    items: list[Union[:class:`Item`, :class:`Asset`]]
         A list of the inventory's items.
     owner: :class:`~steam.User`
         The owner of the inventory.
-    game: :class:`steam.Game`
+    game: Optional[:class:`steam.Game`]
         The game the inventory the game belongs to.
     """
 
     __slots__ = ("game", "items", "owner", "_state", "_total_inventory_count")
 
-    def __init__(self, state: "ConnectionState", data: dict, owner: "BaseUser"):
+    def __init__(self, state: ConnectionState, data: InventoryDict, owner: BaseUser):
         self._state = state
         self.owner = owner
-        self.items: List[Item] = []
+        self.items: list[Items] = []
         self.game: Optional[Game]
         self._update(data)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         attrs = ("owner", "game")
         resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
         return f"<Inventory {' '.join(resolved)}>"
 
-    def __len__(self):
+    def __len__(self) -> int:
         return self._total_inventory_count
 
-    def __iter__(self) -> Iterable[Item]:
+    def __iter__(self) -> Generator[Items, None, None]:
         return (item for item in self.items)
 
-    def __contains__(self, item):
-        if isinstance(item, Asset):
-            return item in self.items
-        return NotImplemented
+    def __contains__(self, item: Asset) -> bool:
+        if not isinstance(item, Asset):
+            raise TypeError("unsupported operand type(s) for 'in':" f" '{item.__class__.__qualname__}' and 'Inventory'")
+        return item in self.items
 
-    def _update(self, data) -> None:
+    def _update(self, data: InventoryDict) -> None:
         try:
-            self.game = Game(app_id=int(data["assets"][0]["appid"]))
+            self.game = Game(id=int(data["assets"][0]["appid"]))
         except KeyError:  # they don't have an inventory in this game
             self.game = None
             self.items = []
@@ -255,68 +320,63 @@ class Inventory:
                 for item in data["descriptions"]:
                     if item["instanceid"] == asset["instanceid"] and item["classid"] == asset["classid"]:
                         item.update(asset)
-                        self.items.append(Item(state=self._state, data=item))
+                        self.items.append(Item(data=item))
                         found = True
                 if not found:
                     self.items.append(Asset(data=asset))
             self._total_inventory_count = data["total_inventory_count"]
 
-    async def update(self) -> "Inventory":
+    async def update(self) -> None:
         """|coro|
-        Re-fetches the :class:`~steam.User`'s inventory.
-
-        Returns
-        ---------
-        :class:`~steam.Inventory`
-            The refreshed inventory.
+        Re-fetches the inventory.
         """
-        data = await self._state.http.get_user_inventory(self.owner.id64, self.game.app_id, self.game.context_id)
+        data = await self._state.http.get_user_inventory(self.owner.id64, self.game.id, self.game.context_id)
         self._update(data)
-        return self
 
-    def filter_items(self, item_name: str, *, limit: int = None) -> List[Item]:
-        """Filters items by name into a list of one type of item.
+    def filter_items(self, *names: str, **kwargs: Any) -> list[Item]:
+        """A helper function that filters and removes items by name from the inventory.
 
         Parameters
         ------------
-        item_name: :class:`str`
-            The item's name to filter for.
+        *names: :class:`str`
+            The names of the items to filter for.
         limit: Optional[:class:`int`]
-            The maximum amount of items to filter.
+            The maximum amount of items to return. Checks from the front of the items.
+
+        Raises
+        -------
+        :exc:`ValueError`
+            You passed a limit and multiple item names.
 
         Returns
         ---------
-        List[:class:`Item`]
-            List of :class:`Item`.
-            Could be an empty if no matching items are found.
-            This also removes the item from the inventory, if possible.
+        list[:class:`Item`]
+            The removed matching items.
         """
-        items = [item for item in self if item.name == item_name]
+        limit: int = kwargs.get("limit")
+        if limit and names:
+            raise ValueError("Cannot pass a limit with multiple items")
+        items = [item for item in self if item.name in names]
         items = items if limit is None else items[:limit]
         for item in items:
             self.items.remove(item)
         return items
 
-    def get_item(self, item_name: str) -> Optional[Item]:
-        """Get an item by name from a :class:`Inventory`.
+    def get_item(self, name: str) -> Optional[Item]:
+        """A helper function that gets and removes an item by name from the inventory.
 
         Parameters
         ----------
-        item_name: :class:`str`
+        name: :class:`str`
             The item to get from the inventory.
 
         Returns
         -------
         Optional[:class:`Item`]
-            Returns the first found item with a matching name.
-            Can be ``None`` if no matching item is found.
-            This also removes the item from the inventory, if possible.
+            Returns the first item found with a matching name. Could be ``None`` if no matching item is found.
         """
-        item = [item for item in self if item.name == item_name]
-        if item:
-            self.items.remove(item[0])
-            return item[0]
-        return None
+        item = self.filter_items(name, limit=1)
+        return item[0] if item else None
 
 
 class TradeOffer:
@@ -325,41 +385,37 @@ class TradeOffer:
 
     Parameters
     ----------
-    item_to_send: Optional[Union[:class:`steam.Item`, :class:`steam.Asset`]]
+    item_to_send: Optional[:class:`steam.Item`]
         The item to send with the trade offer.
-    item_to_receive: Optional[Union[:class:`steam.Item`, :class:`steam.Asset`]]
+    item_to_receive: Optional[:class:`steam.Item`]
         The item to receive with the trade offer.
-    items_to_send: Optional[List[Union[:class:`steam.Item`, :class:`steam.Asset`]]]
+    items_to_send: Optional[list[:class:`steam.Item`]]
         The items you are sending to the other user.
-    items_to_receive: Optional[List[Union[:class:`steam.Item`, :class:`steam.Asset`]]]
+    items_to_receive: Optional[list[:class:`steam.Item`]]
         The items you are sending to the other user.
     token: Optional[:class:`str`]
-        The the trade token used to send trades to users who aren't
-        on the ClientUser's friend's list.
+        The the trade token used to send trades to users who aren't on the ClientUser's friend's list.
     message: Optional[:class:`str`]
          The offer message to send with the trade.
 
     Attributes
     -------------
     partner: Union[:class:`~steam.User`, :class:`~steam.SteamID`]
-        The trade offer partner. This should only ever be a :class:`~steam.SteamID`
-        if the partner's profile is private.
-    items_to_send: Union[List[:class:`Item`], List[:class:`Asset`]]
+        The trade offer partner. This should only ever be a :class:`~steam.SteamID` if the partner's profile is private.
+    items_to_send: Union[list[:class:`Item`]]
         A list of items to send to the partner.
-    items_to_receive: Union[List[:class:`Item`], List[:class:`Asset`]]
+    items_to_receive: Union[list[:class:`Item`]]
         A list of items to receive from the partner.
     state: :class:`~steam.ETradeOfferState`
-        The offer state of the trade for the possible types see
-        :class:`~steam.ETradeOfferState`.
+        The offer state of the trade for the possible types see :class:`~steam.ETradeOfferState`.
     message: :class:`str`
         The message included with the trade offer.
     id: :class:`int`
-        The trade offer id of the trade.
+        The trade's offer ID.
     expires: :class:`datetime.datetime`
         The time at which the trade automatically expires.
     escrow: Optional[:class:`datetime.datetime`]
-        The time at which the escrow will end. Can be None
-        if there is no escrow on the trade.
+        The time at which the escrow will end. Can be ``None`` if there is no escrow on the trade.
     """
 
     __slots__ = (
@@ -382,25 +438,25 @@ class TradeOffer:
         *,
         message: Optional[str] = None,
         token: Optional[str] = None,
-        item_to_send: Optional[Union[Item, Asset]] = None,
-        item_to_receive: Optional[Union[Item, Asset]] = None,
-        items_to_send: Optional[List[Union[Item, Asset]]] = None,
-        items_to_receive: Optional[List[Union[Item, Asset]]] = None,
+        item_to_send: Optional[Items] = None,
+        item_to_receive: Optional[Items] = None,
+        items_to_send: Optional[list[Items]] = None,
+        items_to_receive: Optional[list[Items]] = None,
     ):
-        self.items_to_receive = items_to_receive if items_to_receive else []
-        self.items_to_send = items_to_send if items_to_send else []
+        self.items_to_receive: list[Items] = items_to_receive if items_to_receive else []
+        self.items_to_send: list[Items] = items_to_send if items_to_send else []
         if item_to_receive:
             self.items_to_receive.append(item_to_receive)
         if item_to_send:
             self.items_to_send.append(item_to_send)
-        self.message = message if message is not None else ""
-        self.token = token
+        self.message: str = message if message is not None else ""
+        self.token: Optional[str] = token
         self._has_been_sent = False
-        self.partner: Optional["User"] = None
+        self.partner: Optional[User] = None
         self.state = ETradeOfferState.Invalid
 
     @classmethod
-    async def _from_api(cls, state: "ConnectionState", data: dict) -> "TradeOffer":
+    async def _from_api(cls, state: ConnectionState, data: TradeOfferDict) -> TradeOffer:
         from .abc import SteamID
 
         trade = cls()
@@ -412,12 +468,12 @@ class TradeOffer:
         )  # the account is private :(
         return trade
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         attrs = ("id", "state", "partner")
         resolved = [f"{attr}={getattr(self, attr, None)!r}" for attr in attrs]
         return f"<TradeOffer {' '.join(resolved)}>"
 
-    def _update(self, data) -> None:
+    def _update(self, data: TradeOfferDict) -> None:
         self.message = data.get("message") or None
         self.id = int(data["tradeofferid"])
         expires = data.get("expiration_time")
@@ -425,15 +481,12 @@ class TradeOffer:
         self.expires = datetime.utcfromtimestamp(expires) if expires else None
         self.escrow = datetime.utcfromtimestamp(escrow) if escrow else None
         self.state = ETradeOfferState(data.get("trade_offer_state", 1))
-        self.items_to_send = [Item(state=self._state, data=item) for item in data.get("items_to_give", [])]
-        self.items_to_receive = [Item(state=self._state, data=item) for item in data.get("items_to_receive", [])]
+        self.items_to_send = [Item(data=item) for item in data.get("items_to_give", [])]
+        self.items_to_receive = [Item(data=item) for item in data.get("items_to_receive", [])]
         self._is_our_offer = data.get("is_our_offer", False)
 
-    def __eq__(self, other):
-        if isinstance(other, TradeOffer):
-            if self._has_been_sent and other._has_been_sent:
-                return self.id == other.id
-        return False
+    def __eq__(self, other: Any) -> bool:
+        return isinstance(other, TradeOffer) and self._has_been_sent and other._has_been_sent and self.id == other.id
 
     async def confirm(self) -> None:
         """|coro|
@@ -479,8 +532,7 @@ class TradeOffer:
                 try:
                     await self.confirm()
                 except ConfirmationError:
-                    await asyncio.sleep(tries * 2)
-                    continue
+                    break
 
     async def decline(self) -> None:
         """|coro|
@@ -514,7 +566,7 @@ class TradeOffer:
             raise ClientException("Offer wasn't created by the ClientUser and therefore cannot be canceled")
         await self._state.http.cancel_user_trade(self.id)
 
-    async def counter(self, trade: "TradeOffer") -> None:
+    async def counter(self, trade: TradeOffer) -> None:
         """|coro|
         Counters a trade offer from an :class:`User`.
 
@@ -535,18 +587,18 @@ class TradeOffer:
 
         to_send = [item.to_dict() for item in trade.items_to_send]
         to_receive = [item.to_dict() for item in trade.items_to_receive]
-        resp = await self._state.http.send_counter_trade_offer(
-            self.id, self.partner.id64, self.partner.id, to_send, to_receive, trade.token, trade.message,
+        resp = await self._state.http.send_trade_offer(
+            self.partner, to_send, to_receive, trade.token, trade.message, trade_id=self.id
         )
         if resp.get("needs_mobile_confirmation", False):
             await self._state.get_and_confirm_confirmation(int(resp["tradeofferid"]))
 
     def is_gift(self) -> bool:
-        """:class:`bool`: Checks if an offer is a gift to the ClientUser"""
+        """:class:`bool`: Helper method that checks if an offer is a gift to the :class:`~steam.ClientUser`"""
         return self.items_to_receive and not self.items_to_send
 
     def is_our_offer(self) -> bool:
-        """:class:`bool`: Whether the offer was created by the ClientUser."""
+        """:class:`bool`: Whether the offer was created by the :class:`~steam.ClientUser`."""
         return self._is_our_offer
 
     def _check_active(self) -> None:
